@@ -3,13 +3,30 @@
 
 #include "config.h"
 #include "meta.h"
+#include "acbuf.h"
+
+#ifdef DEBUG
+
+struct t_logger
+{
+	t_logger(const char *szFuncName); // starts the logger, shifts stack depth
+	~t_logger();
+	tSS & GetFmter();
+	void Write(const char *pFile=NULL, unsigned int nLine=0);
+private:
+	tSS m_strm;
+	pthread_t m_id;
+	unsigned int m_nLevel;
+	const char * m_szName;
+};
+#endif
+
 
 namespace aclog {
 
       bool open();
-      void close();
-      void reopen();
-      void transfer(bool bDirIn, uint64_t nCount, const char *szClient, const char *szPath);
+      void close(bool bReopen);
+      void transfer(char cLogType, uint64_t nCount, const char *szClient, const char *szPath);
       void err(const char *msg, const char *client=NULL);
       void misc(const MYSTD::string & sLine);
       inline void err(const MYSTD::string msg) { err(msg.c_str()); }
@@ -37,17 +54,17 @@ namespace aclog {
     	 // tRowData & operator=(const tRowData &a);
       };
 
-      void GetStats(MYSTD::vector<tRowData> & out);
+      MYSTD::string GetStatReport();
       
-      struct errnoFmter {
+      struct errnoFmter : public MYSTD::string {
     	  errnoFmter();
-    	  const char *msg;
       };
 }
 
 //#define TIMEFORMAT "%a %d/%m"
 #define TIMEFORMAT "%Y-%m-%d %H:%M"
 
+
 #endif
 
 
