@@ -9,7 +9,8 @@
 #define BGTASK_H_
 
 #include "maintenance.h"
-#include <stdio.h>
+#include <iostream>
+#include <fstream>
 
 class bgtask : public maintenance
 {
@@ -31,16 +32,10 @@ protected:
 	// to be implemented by subclasses
 	virtual void Action(const MYSTD::string &) =0;
 
+	using maintenance::SendChunk;
 
-	/// pass it to the base class but store a copy to a file
-
-	virtual void SendChunk(const char *data, size_t len);
-	/*
-	 * In theory, it shouldn't be necessary because there is base
-	 * implementation which is virtual... but for some STUPID REASON
-	 * the compiler goes nuts and refuses to consider using it
-	 * */
-	void SendChunk(const MYSTD::string &x) { SendChunk(x.c_str(), x.size()); }
+	/// pass it to the base class but store a copy to some log file
+	virtual void SendChunk(const char *data, size_t size);
 
 	void DumpLog(time_t id);
 
@@ -48,7 +43,7 @@ protected:
 
 private:
 
-	FILE *m_repFileStream;
+	MYSTD::ofstream m_reportStream;
 };
 
 #endif /* BGTASK_H_ */
