@@ -29,9 +29,9 @@ class con {
       void SignalStop();
 #endif
       
-      void LogDataCounts(MYSTD::string & file, 
+      void LogDataCounts(const MYSTD::string & file,
     		  const char *xff,
-    		  off_t countIn, off_t countOut);
+    		  off_t countIn, off_t countOut, bool bFileIsError);
       
    private:
 	   con& operator=(const con&);// { /* ASSERT(!"Don't copy con objects"); */ };
@@ -62,9 +62,16 @@ class con {
       MYSTD::string m_sClientHost;
       header *m_pTmpHead;
       
-      MYSTD::string m_sLoggedFile, m_sLoggedClient;
-      off_t m_nLastIn, m_nLastOut;
-      void _LogFlush();
+      struct __tlogstuff
+      {
+    	  MYSTD::string file, client;
+    	  off_t sumIn, sumOut;
+    	  bool bFileIsError;
+    	  void write();
+    	  void reset(const MYSTD::string &pNewFile, const MYSTD::string &pNewClient, bool bIsError);
+    	  inline __tlogstuff() : sumIn(0), sumOut(0), bFileIsError(false) {}
+      } logstuff;
+
 };
 
 #endif
