@@ -4,9 +4,9 @@
 #include "meta.h"
 #include "fileitem.h"
 #include "acfg.h"
+#include "acbuf.h"
 
 class dlcon;
-class acbuf;
 
 class tDlJob
 {
@@ -44,7 +44,7 @@ public:
 	bool FindConfig();
 	
 	// needs connectedHost, blacklist, output buffer from the parent, proxy mode?
-	bool AppendRequest(MYSTD::string sForThisHostOnly, MYSTD::string &head);
+	bool AppendRequest(MYSTD::string sForThisHostOnly, tSS &head);
 
 	inline bool HasStarted() { return m_DlState>STATE_GETHEADER; }
 
