@@ -53,13 +53,13 @@ public:
 protected:
 
 	//friend class showexp;
-	tStrSet m_volatileFiles;
+	tStrSet m_indexFiles;
 
-	void _UpdateVolatileFiles();
-	void _UpdateWithDiffs(dlcon &dl, tStrSet & listGuessedLocationsToTry);
+	void UpdateIndexFiles();
 	void _BusyDisplayLogs();
 	void _Usermsg(MYSTD::string m);
 	virtual void _HandlePkgEntry(const tRemoteFileInfo &entry);
+	void AddIFileCandidate(const MYSTD::string &sFile);
 	
 	/*!
 	 * As the name saids, processes all index files and calls a callback
@@ -71,11 +71,10 @@ protected:
 	 * having strict path checking disabled, though.
 	 *   
 	 * */
-	void _ParseVolatileFilesAndHandleEntries(tStrSet *pDupeCatcher=NULL);
+	void _ProcessSeenIndexFiles(tStrSet *pDupeCatcher=NULL);
 	bool _UpdateOne(dlcon &dl, const MYSTD::string & sFilePath, MYSTD::string &sErrOut, tDlJobHints *p=NULL);
 	void _InjectBz2ed(const MYSTD::string & sTargetBasePath, 
 			const MYSTD::string & sSrcTmpPath, const tStrSet & goodChecklist);
-	bool _DownloadOne(dlcon &dl, const MYSTD::string & sFilePath, MYSTD::string &sErrOut);
 	virtual void _LoadIgnoreList();
 	virtual bool _CanBeIgnored(const MYSTD::string &);
 		
@@ -100,6 +99,10 @@ private:
 	void _RemoveAndStoreStatus(bool bPurgeNow);
 	void _LoadTrashMapFromFile(bool bForceInsert);
 	void _PurgeMaintLogs();
+	void _UpdateWithDiffs(dlcon &dl, tStrSet & listGuessedLocationsToTry);
+	bool _ParseAndProcessIndexFile(const MYSTD::string &sPath,
+			const MYSTD::string &sDirname, const MYSTD::string &sDebBaseDir,
+			bool *pbProbeType);
 	
 	unsigned int m_nProgIdx, m_nProgTold;
 
