@@ -11,6 +11,9 @@
 #ifdef HAVE_ZLIB
 #include <zlib.h>
 #endif
+#ifdef HAVE_LZMA
+#include <lzma.h>
+#endif
 
 /*!
  * Helper class used to read files.
@@ -38,9 +41,8 @@ public:
 	//! Returns lines when beginning with non-space, otherwise empty string. 
 	//! @return False on errors.
 	bool GetOneLine(MYSTD::string & sOut);
-	MYSTD::string GetPositionDescription();
-	// void Close();
-	bool CheckGoodState(bool bTerminateOnErrors);
+	MYSTD::string GetPositionDescription() { return (tSS() << m_sOrigName << ":" << m_nCurLine); }
+	bool CheckGoodState(bool bTerminateOnErrors) const;
 	
 	static bool GetMd5Sum(const MYSTD::string & sFileName, uint8_t out[], 
 			bool bTryUnpack, off_t &scannedSize);
@@ -80,6 +82,9 @@ private:
 #ifdef HAVE_ZLIB
 	z_stream m_zStream;
 #endif
+#ifdef HAVE_LZMA
+	lzma_stream m_xzStream;
+#endif
 	
 	// not to be copied
 	filereader& operator=(const filereader&);
@@ -87,8 +92,6 @@ private:
 
 };
 
-namespace Meta
-{
 extern uint_fast16_t hexmap[];
 
 inline bool CsEqual(CUCHAR a[], uint8_t b[], unsigned short binLen)
@@ -100,7 +103,6 @@ inline bool CsEqual(CUCHAR a[], uint8_t b[], unsigned short binLen)
 	}
 	return true;
 };
-}
 
 #endif // __FILEREADER_H
 
