@@ -28,7 +28,7 @@ class header {
     	  AUTHORIZATION,		// 10
     	  XFORWARDEDFOR,
     	  LOCATION,
-
+    	  CONTENT_TYPE,
     	  // unreachable entry and size reference
     	  HEADPOS_MAX
       };
@@ -45,8 +45,7 @@ class header {
       header& operator=(const header&); 
       ~header();
       
-      static MYSTD::string GetInfoHeaders();
-      //header(const MYSTD::string & sFrontLine);
+      static MYSTD::string GenInfoHeaders();
 
       /*!
        * Read buffer to parse one string. Optional offset where to begin to
@@ -65,17 +64,20 @@ class header {
       
       void set(eHeadPos, const MYSTD::string &value);
       void set(eHeadPos, const char *val);
-      void set(eHeadPos, long nValue);
+      void set(eHeadPos, const char *s, size_t len);
+      void set(eHeadPos, off_t nValue);
       void del(eHeadPos);
       //MYSTD::string getLike(const MYSTD::string &pattern);
       //bool getUri(MYSTD::string & sRet);
-      inline int getStatus() const { return atoi(frontLine.c_str()+8); };
+      inline int getStatus() const { return frontLine.length()>9 ? atoi(frontLine.c_str()+8) : 500; };
       //void SetStatusLine(MYSTD::string sLine);
       //MYSTD::string const & get(const MYSTD::string &key) const;
       //MYSTD::string as_string(bool bTerminateIt) const;
       //void strip();
       void clear();
       
+      MYSTD::string ToString();
+
    private:
 	   int Load(const char *src, UINT length);
 };
