@@ -4,6 +4,7 @@
 
 #include "config.h"
 #include "header.h"
+#include "acbuf.h"
 #include <sys/types.h>
 
 class fileitem;
@@ -18,14 +19,13 @@ class job {
 
 	   typedef enum
 	{
-		R_UNDEFINED = -1, R_DONE = 0, R_AGAIN = 1, R_DISCON = 2, R_NOTFORUS = 3
+		R_DONE = 0, R_AGAIN = 1, R_DISCON = 2, R_NOTFORUS = 3
 	} eJobResult;
 
 	   typedef enum {
-	   	STATE_FRESH,
+	   	STATE_SEND_MAIN_HEAD,
 	   	STATE_HEADER_SENT,
 	   	STATE_SEND_PLAIN_DATA,
-	   	STATE_FAILMODE,
 	   	STATE_SEND_CHUNK_HEADER,
 	   	STATE_SEND_CHUNK_DATA,
 	   	STATE_TODISCON,
@@ -48,39 +48,40 @@ class job {
        */
       eJobResult SendData(int confd, int nAllJobCount);
   
-      static void FixUri(MYSTD::string &sUri);
-      
    private:
       
 	  int m_filefd;
 	  con *m_pParentCon;
       
-      bool m_bChunkMode:1;
-      //bool m_bCloseCon:1;
-      bool m_bIsHttp11:1;
+      bool m_bChunkMode;
+      bool m_bClientWants2Close;
+      bool m_bIsHttp11;
       
       eJobState m_state, m_backstate;
       
-      MYSTD::string m_sErrorMsgBuffer;
-      MYSTD::string m_sPrependBuf;
+      //MYSTD::string m_sErrorMsgBuffer;
+      tSS m_sendbuf;
       MYSTD::string m_sFileLoc; // local_relative_path_to_file
       MYSTD::string m_sMaintCmd;
+      MYSTD::string m_sOrigUrl; // local SAFE copy of the originating source
       
       header *m_pReqHead; // copy of users requests header
-      header m_RespHead; // template of the response header, e.g. as found in cache
       
       tFileItemPtr m_pItem;
       off_t m_nSendPos, m_nRangeLast;
       off_t m_nAllDataCount;
       
-      unsigned int m_nChunkRemainingBytes;
+      unsigned long m_nChunkRemainingBytes;
       USHORT m_type; // file type, bit mask
       
       job(const job&);
       job & operator=(const job&);
 
-      eJobResult BuildAndEnqueHeader(const char * &szErrRet, const FiStatus &fistate, const off_t &nGooddataSize);
+      const char * BuildAndEnqueHeader(const FiStatus &fistate, const off_t &nGooddataSize, header& respHead);
       FiStatus _SwitchToPtItem(const MYSTD::string &fileLoc);
+      void SetErrorResponse(const char * errorLine, int nMinUserLogLevel=SPAMLEVEL);
+      void HandleLocalDownload(const MYSTD::string &visPath,
+    			const MYSTD::string &fsBase, const MYSTD::string &fsSubpath);
 };
 
 #endif
