@@ -3,6 +3,8 @@
 
 #include "config.h"
 #include "meta.h"
+#include "sockio.h"
+#include "acbuf.h"
 
 class maintenance
 {
@@ -16,8 +18,8 @@ public:
 	virtual void Run(const MYSTD::string &cmd)=0;
 
 protected:
-	
-	virtual void SendChunk(const MYSTD::string &x);
+	inline void SendChunk(const MYSTD::string &x) { SendChunk(x.data(), x.size()); }
+	inline void SendChunk(const tSS &x){ SendChunk(x.data(), x.size()); }
 	virtual void SendChunk(const char *data, size_t size);
 	bool SendRawData(const char *data, size_t len, int flags);
 	virtual void EndTransfer();
