@@ -39,7 +39,7 @@ private:
 	/* tFprCacheMap m_cacheMap;*/
 	
 
-	bool m_bPickIfiles;
+	bool m_bLookForIFiles;
 	MYSTD::string m_sSrcPath;
 		
 	bool LinkOrCopy(const MYSTD::string &from, const MYSTD::string &to);
