@@ -5,34 +5,21 @@
 #include "config.h"
 
 #include "acbuf.h"
-#include <errno.h>
+#include "fileio.h"
 #include <unistd.h>
-#include <errno.h>
-#include <sys/stat.h>
 
 bool acbuf::init(unsigned int c) {
 	if (m_buf && m_nCapacity==c)
 	{
-		ldbg("buffer allocated before");
+		//ldbg("buffer allocated before");
 	}
 	else
 	{
-		m_nCapacity=c;
-		if (m_buf)
-		{
-			delete [] m_buf;
-			m_buf=NULL;
-		}
-		MYTRY
-		{
-			m_buf=new char[c+1];
-		}
-		MYCATCH(MYSTD::bad_alloc)
-		{
-			return false;
-		}
+		free(m_buf);
+		m_buf=(char*) malloc(c+1);
 		if(!m_buf)
 			return false;
+		m_nCapacity=c;
 	}
     m_buf[c]=0; // terminate to make string operations safe
     r=w=0;
@@ -94,3 +81,4 @@ int acbuf::sysread(int fd) {
         w+=n;
     return(n);
 }
+
