@@ -6,15 +6,14 @@
 #include "acfg.h"
 #include "meta.h"
 #include "filereader.h"
+#include "fileio.h"
 
-#include <errno.h>
 #include <regex.h>
 
-
 #include <iostream>
 #include <fstream>
 #include <string>
-#include <list>
+#include <deque>
 #include <map>
 #include <algorithm>
 
@@ -33,7 +32,10 @@ namespace acfg {
 char alphabet[65] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 string sPopularPath("/debian/");
 
-string dontcache, proxy, dontcacherq, dontcachetgt;
+string dontcache, dontcacherq, dontcachetgt;
+
+tStringMap localdirs;
+NoCaseStringMap mimemap;
 
 MapNameToString n2sTbl[] = {
 		{  "Port", 			&port , 0}
@@ -41,7 +43,7 @@ MapNameToString n2sTbl[] = {
 		,{ "LogDir", 	&logdir , 0}
 		,{ "SocketPath", 	&fifopath, 0}
 		,{ "PidFile", 	&pidfile, 0}
-		,{ "Proxy",		&proxy, 0}
+		//,{ "Proxy",		&proxy, 0}
 		,{ "ReportPage",&reportpage, 0}
 		,{ "VfilePattern", &vfilepat, 0}
 		,{ "PfilePattern", &pfilepat, 0}
@@ -203,11 +205,10 @@ inline void _AddRemapInfo(bool bAsBackend, const string & token,
 		if (sPath.empty())
 			BARF("Bad file spec for repname, file:?");
 		
-		if (sPath[0]!=cPathSep)
-			sPath.insert(0, confdir+sPathSep);
+		MakeAbsolutePath(sPath, confdir);
 		
-		tStrVec srcs=ExpandFilePattern(sPath, true);
-		for(tStrVecIterConst it=srcs.begin(); it!=srcs.end(); it++)
+		tStrDeq srcs=ExpandFilePattern(sPath, true);
+		for(tStrDeq::const_iterator it=srcs.begin(); it!=srcs.end(); it++)
 		{
 			if (bAsBackend)
 				_ReadBackendsFiles(*it, repname);
@@ -217,7 +218,55 @@ inline void _AddRemapInfo(bool bAsBackend, const string & token,
 	}
 }
 
-bool SetOption(const MYSTD::string &sLine, bool bQuiet)
+inline string _GetBase64Auth(const string & sUserColonPass)
+{
+	int cols, bits, c, char_count;
+
+	char_count = 0;
+	bits = 0;
+	cols = 0;
+	tStrPos pos(0);
+	string out;
+	while ( pos<sUserColonPass.size())
+	{
+		c=sUserColonPass[pos++];
+		bits += c;
+		char_count++;
+		if (char_count == 3)
+		{
+			out+=(alphabet[bits >> 18]);
+			out+=(alphabet[(bits >> 12) & 0x3f]);
+			out+=(alphabet[(bits >> 6) & 0x3f]);
+			out+=(alphabet[bits & 0x3f]);
+			cols += 4;
+			bits = 0;
+			char_count = 0;
+		}
+		else
+		{
+			bits <<= 8;
+		}
+	}
+	if (char_count != 0)
+	{
+		bits <<= 16 - (8 * char_count);
+		out+=(alphabet[bits >> 18]);
+		out+=(alphabet[(bits >> 12) & 0x3f]);
+		if (char_count == 1)
+		{
+			out+=('=');
+			out+=('=');
+		}
+		else
+		{
+			out+=(alphabet[(bits >> 6) & 0x3f]);
+			out+=('=');
+		}
+	}
+	return out;
+}
+
+bool SetOption(const string &sLine, bool bQuiet)
 {
 	string::size_type posCol = sLine.find(":");
 	string::size_type posEq = sLine.find("=");
@@ -280,7 +329,87 @@ bool SetOption(const MYSTD::string &sLine, bool bQuiet)
 			cerr << "Bad value for " << key << " option or found trailing garbage: " << pEnd <<endl;
 			return false;
 		}
-				
+	}
+
+	else if(0==strcasecmp(key.c_str(), "Proxy"))
+	{
+		if (!proxy_info.SetHttpUrl(value))
+		{
+			cerr << "Invalid proxy specification, aborting..." << endl;
+			exit(EXIT_FAILURE);
+		}
+		tStrPos pos = proxy_info.sHost.find('@');
+		if (stmiss != pos)
+		{
+			proxy_info.sPath = string("Proxy-Authorization: Basic ")
+					+ _GetBase64Auth(value.substr(0, pos)) + "\r\n";
+			proxy_info.sHost.erase(0, pos + 1);
+		}
+		else
+			proxy_info.sPath.clear(); // no auth at all, stop confusion
+
+		if (proxy_info.sHost.empty())
+		{
+			cerr << "Invalid proxy specification, aborting..." << endl;
+			exit(EXIT_FAILURE);
+		}
+		if (proxy_info.sPort.empty())
+		{
+			if (proxy_info.sPath.empty()) // guess unless there is any risk...
+			{
+				cerr << "Warning, unknown proxy port, assuming 80." << endl;
+				proxy_info.sPort = "80";
+			}
+			else
+			{
+				cerr << "Error, unknown proxy port!" << endl;
+				exit(EXIT_FAILURE);
+			}
+		}
+	}
+	else if(0==strcasecmp(key.c_str(), "LocalDirs"))
+	{
+		tStrVec tokens;
+		Tokenize(value, ";", tokens);
+		for(tStrVecIter it=tokens.begin(); it!=tokens.end(); it++)
+		{
+			trimString(*it);
+			tStrPos pos = it->find_first_of(SPACECHARS);
+			if(stmiss == pos)
+			{
+				cerr << "Cannot map " << *it << ", needed format: virtualdir realdir, ignoring it";
+				continue;
+			}
+			string from=it->substr(0, pos);
+			trimString(from, "/");
+			string what=it->substr(pos);
+			trimString(what, SPACECHARS "'\"");
+			if(what.empty())
+			{
+				cerr << "Unsupported target of " << from << ": " << what << ", ignoring it" << endl;
+				continue;
+			}
+			localdirs[from]=what;
+		}
+
+		filereader mreader;
+		tStrVec toks;
+		string line;
+		for(mreader.OpenFile("/etc/mime.types"); mreader.CheckGoodState(false); )
+		{
+			if(!mreader.GetOneLine(line))
+				break;
+			if(IsValidButIrrelevantLine(line))
+				continue;
+
+			if(Tokenize(line, SPACECHARS, toks, false)>1)
+			{
+				for(UINT i=1; i<toks.size();i++)
+					mimemap[toks[i]]=toks[0];
+			}
+		}
+
+		return !localdirs.empty();
 	}
 	else if(0==strncasecmp(key.c_str(), "Remap-", 6))
 	{
@@ -319,7 +448,7 @@ bool SetOption(const MYSTD::string &sLine, bool bQuiet)
 
 
 //const string * GetVnameForUrl(string path, string::size_type * nMatchLen)
-const string * GetRepNameAndPathResidual(const tHttpUrl & in, MYSTD::string & sRetPathResidual)
+const string * GetRepNameAndPathResidual(const tHttpUrl & in, string & sRetPathResidual)
 {
 	sRetPathResidual.clear();
 	
@@ -567,62 +696,14 @@ void ReadConfigDirectory(const char *szPath)
 	confdir=buf; // pickup the last config directory
 
 #if defined(HAVE_WORDEXP) || defined(HAVE_GLOB)
-	tStrVec srcs=ExpandFilePattern(confdir+SZPATHSEP"*.conf", true);
-	for(tStrVecIterConst it=srcs.begin(); it!=srcs.end(); it++)
+	tStrDeq srcs=ExpandFilePattern(confdir+SZPATHSEP"*.conf", true);
+	for(tStrDeq::const_iterator it=srcs.begin(); it!=srcs.end(); it++)
 		_ReadMainConfiguration(*it);
 #else
 	_ReadMainConfiguration(confdir+SZPATHSEP"acng.conf");
 #endif
 }
 
-string _GetBase64Auth(const string & sUserColonPass)
-{
-	int cols, bits, c, char_count;
-
-	char_count = 0;
-	bits = 0;
-	cols = 0;
-	tStrPos pos(0);
-	MYSTD::string out;
-	while ( pos<sUserColonPass.size())
-	{
-		c=sUserColonPass[pos++];
-		bits += c;
-		char_count++;
-		if (char_count == 3)
-		{
-			out+=(alphabet[bits >> 18]);
-			out+=(alphabet[(bits >> 12) & 0x3f]);
-			out+=(alphabet[(bits >> 6) & 0x3f]);
-			out+=(alphabet[bits & 0x3f]);
-			cols += 4;
-			bits = 0;
-			char_count = 0;
-		}
-		else
-		{
-			bits <<= 8;
-		}
-	}
-	if (char_count != 0)
-	{
-		bits <<= 16 - (8 * char_count);
-		out+=(alphabet[bits >> 18]);
-		out+=(alphabet[(bits >> 12) & 0x3f]);
-		if (char_count == 1)
-		{
-			out+=('=');
-			out+=('=');
-		}
-		else
-		{
-			out+=(alphabet[(bits >> 6) & 0x3f]);
-			out+=('=');
-		}
-	}
-	return out;
-}
-
 void PostProcConfig() 
 {
 	if(stupidfs)
@@ -663,7 +744,7 @@ void PostProcConfig()
 	}
 #endif
 	
-   if(cachedir.empty() || cachedir[0] != sPathSep[0]) {
+   if(cachedir.empty() || cachedir[0] != CPATHSEP) {
       cerr << "Cache directory unknown or not absolute, terminating..." <<endl;
       exit(EXIT_FAILURE);
    }
@@ -678,51 +759,17 @@ void PostProcConfig()
 	   acfg::tpthreadmax = numeric_limits<int>::max();
 	   
    // get rid of duplicated and trailing slash(es)
-   tStrVec tmp;
-   Tokenize(cachedir, sPathSep.c_str(), tmp);
-   Join(cachedir, sPathSep, tmp);
-   
-   if(!pidfile.empty() && pidfile.at(0) != cPathSep)
+	for(tStrPos pos; stmiss != (pos = cachedir.find(SZPATHSEP SZPATHSEP )); )
+		cachedir.erase(pos, 1);
+
+   if(!pidfile.empty() && pidfile.at(0) != CPATHSEP)
    {
 	   cerr << "Pid file path must be absolute, terminating..."  <<endl;
 	         exit(EXIT_FAILURE);
    }
    
-   if(! proxy.empty()) {
-	   if(!proxy_info.SetHttpUrl(proxy))
-	   {
-		   cerr << "Invalid proxy specification, aborting..." << endl;
-		   exit(EXIT_FAILURE);
-	   }
-	   
-	   tStrPos pos=proxy_info.sHost.find('@');
-	   if(stmiss != pos)
-	   {
-		   proxy_info.sPath=string("Proxy-Authorization: Basic ")+_GetBase64Auth(proxy.substr(0, pos))+"\r\n";
-		   proxy_info.sHost.erase(0,pos+1);
-	   }
-     else
-        proxy_info.sPath.clear(); // no auth at all, stop confusion
-
-	   if(proxy_info.sHost.empty())
-	   {
-		   cerr << "Invalid proxy specification, aborting..." << endl;
-		   exit(EXIT_FAILURE);
-	   }
-	   if (proxy_info.sPort.empty())
-		{
-			if (proxy_info.sPath.empty()) // guess unless there is any risk...
-			{
-				cerr << "Warning, unknown proxy port, assuming 80." <<endl;
-				proxy_info.sPort="80";
-			}
-			else
-			{
-				cerr << "Error, unknown proxy port!"<<endl;
-				exit(EXIT_FAILURE);
-			}
-	   }
-   }
+   if(!acfg::agentname.empty())
+	   acfg::agentheader=string("User-Agent: ")+acfg::agentname + "\r\n";
    
    if(!adminauth.empty())
 	   adminauth=string("Basic ")+_GetBase64Auth(adminauth);
@@ -751,27 +798,35 @@ void PostProcConfig()
 	   exit(EXIT_FAILURE);
    }
 
-#ifdef DEBUG
-   if (acfg::debug>8)
+   if (acfg::debug>3)
 	{
 		for (UINT i=0; i<_countof(n2sTbl); i++)
-		{
-			if(! n2sTbl[i].ptr)
-				continue;
+			if(n2sTbl[i].ptr)
+				cout << n2sTbl[i].name << " = " << *(n2sTbl[i].ptr) <<endl;
 
-			string tmp, *val=n2sTbl[i].ptr;
-			cout << n2sTbl[i].name << " = " << *val <<endl;
-			// escaped version
-			for (const char *p=val->c_str(); *p; p++)
-				tmp.append('\\'==*p ? 2 : 1, *p);
-			cout << n2sTbl[i].name << " = " << tmp << endl;
+		if (acfg::debug > 4)
+		{
+			cerr << "escaped version:" << endl;
+			for (UINT i = 0; i < _countof(n2sTbl); i++)
+				if (n2sTbl[i].ptr)
+				{
+					cout << n2sTbl[i].name << " = ";
+					for (const char *p = n2sTbl[i].ptr->c_str(); *p; p++)
+						if('\\' == *p)
+							cout << "\\\\";
+						else
+							cout << *p;
+					cout <<endl;
+				}
 		}
+
 		for (UINT i=0; i<_countof(n2iTbl); i++)
 			if(n2iTbl[i].ptr)
 				cout << n2iTbl[i].name << " = \"" << *(n2iTbl[i].ptr) << "\"\n";
 	}
-#else
-   if(acfg::debug>2)
+
+#ifndef DEBUG
+   if(acfg::debug>=5)
 	   cout << "\n\nAdditional debugging information not compiled in.\n\n";
 #endif
    
@@ -793,7 +848,7 @@ void PostProcConfig()
 namespace rechecks
 {
 regex_t reIfiles, rePfiles, reWfiles;
-MYSTD::vector<regex_t> vecReqPatters, vecTgtPatterns;
+deque<regex_t> vecReqPatters, vecTgtPatterns;
 
 bool CompileExpressions()
 {
@@ -809,26 +864,29 @@ bool CompileExpressions()
     return true;
 }
 
-eFileKind GetFiletype(const MYSTD::string & in) {
+eFileKind GetFiletype(const string & in) {
 	LOGSTART("rechecks::getFiletype");
     dbgline;
-    if(!regexec(&rePfiles, in.c_str(), 0, NULL, 0))
-        return FILE_PKG;
-    dbgline;
     if(!regexec(&reIfiles, in.c_str(), 0, NULL, 0))
         return FILE_INDEX;
     dbgline;
+    if(!regexec(&rePfiles, in.c_str(), 0, NULL, 0))
+        return FILE_PKG;
+    dbgline;
     return FILE_INVALID;
 }
 
-bool MatchWhitelist(const MYSTD::string & in)
+bool MatchWhitelist(const string & in)
 {
-	return !regexec(&reWfiles, in.c_str(), 0, NULL, 0);
+	LOGSTART("MatchWhiteList");
+	bool bPersistent=!regexec(&reWfiles, in.c_str(), 0, NULL, 0);
+	LOG(in <<" is " << (bPersistent ? "persistent" : "mortal") <<  " file");
+	return bPersistent;
 }
 
-inline bool CompileUncachedRex(const MYSTD::string & token, bool bIsTgtPattern, bool bRecursiveCall)
+inline bool CompileUncachedRex(const string & token, bool bIsTgtPattern, bool bRecursiveCall)
 {
-	MYSTD::vector<regex_t> & patvec = bIsTgtPattern ? vecTgtPatterns : vecReqPatters;
+	deque<regex_t> & patvec = bIsTgtPattern ? vecTgtPatterns : vecReqPatters;
 
 	if (0!=token.compare(0, 5, "file:")) // pure pattern
 	{
@@ -842,11 +900,10 @@ inline bool CompileUncachedRex(const MYSTD::string & token, bool bIsTgtPattern,
 		if (sPath.empty())
 			BARF("Bad file spec for repname, file:?");
 
-		if (sPath[0]!=cPathSep)
-			sPath.insert(0, acfg::confdir+sPathSep);
+		MakeAbsolutePath(sPath, acfg::confdir);
 
-		tStrVec srcs = ExpandFilePattern(sPath, true);
-		for(tStrVecIterConst it=srcs.begin(); it!=srcs.end(); it++)
+		tStrDeq srcs = ExpandFilePattern(sPath, true);
+		for(tStrDeq::const_iterator it=srcs.begin(); it!=srcs.end(); it++)
 		{
 			filereader reader;
 			if(!reader.OpenFile(*it))
@@ -869,7 +926,7 @@ inline bool CompileUncachedRex(const MYSTD::string & token, bool bIsTgtPattern,
 }
 
 
-bool CompileUncExpressions(const MYSTD::string & req, const MYSTD::string & tgt)
+bool CompileUncExpressions(const string & req, const string & tgt)
 {
 	tStrVec pats;
 	Tokenize(req, SPACECHARS, pats, false);
@@ -890,10 +947,10 @@ bool CompileUncExpressions(const MYSTD::string & req, const MYSTD::string & tgt)
 
 
 
-bool MatchUncacheableRequest(const MYSTD::string & in)
+bool MatchUncacheableRequest(const string & in)
 {
 	LOGSTART2("MatchUncacheableRequest", in << " against " << vecReqPatters.size() << " patterns");
-	for(vector<regex_t>::const_iterator it=vecReqPatters.begin();
+	for(deque<regex_t>::const_iterator it=vecReqPatters.begin();
 			it!=vecReqPatters.end(); it++)
 	{
 		if(!regexec(& (*it), in.c_str(), 0, NULL, 0))
@@ -902,9 +959,9 @@ bool MatchUncacheableRequest(const MYSTD::string & in)
 	return false;
 }
 
-bool MatchUncacheableTarget(const MYSTD::string &in)
+bool MatchUncacheableTarget(const string &in)
 {
-	for(vector<regex_t>::const_iterator it=vecTgtPatterns.begin();
+	for(deque<regex_t>::const_iterator it=vecTgtPatterns.begin();
 			it!=vecTgtPatterns.end(); it++)
 	{
 		if(!regexec(& (*it), in.c_str(), 0, NULL, 0))
@@ -915,7 +972,7 @@ bool MatchUncacheableTarget(const MYSTD::string &in)
 
 }
 
-void mkbasedir(const MYSTD::string & path)
+void mkbasedir(const string & path)
 {
     for(UINT pos=0; pos<path.size(); pos=path.find("/", pos+1))
     {
