@@ -15,8 +15,10 @@
 #include <unistd.h>
 
 #include <vector>
+#include <deque>
 #include <string.h>
 #include <iostream>
+#include <fstream>
 #include <meta.h>
 
 using namespace MYSTD;
@@ -24,47 +26,53 @@ using namespace MYSTD;
 namespace aclog
 {
 
-FILE *fErr(NULL), *fStat(NULL);
+ofstream fErr, fStat;
 static pthread_mutex_t mx = PTHREAD_MUTEX_INITIALIZER;
 
 bool open()
 {
+	// only called in the beginning or when reopening, already locked...
+	// lockguard g(&mx);
+
 	if(acfg::logdir.empty())
 		return true;
 	
 	string apath(acfg::logdir+"/apt-cacher.log"), epath(acfg::logdir+"/apt-cacher.err");
 	
-	fErr = fopen(epath.c_str(), "a");
-	fStat = fopen(apath.c_str(), "a");
+	if(fErr.is_open())
+		fErr.close();
+	if(fStat.is_open())
+		fStat.close();
 
-	return fStat && fErr;
+	fErr.open(epath.c_str(), ios::out | ios::app);
+	fStat.open(apath.c_str(), ios::out | ios::app);
+
+	return fStat.is_open() && fErr.is_open();
 }
 
 
-void transfer(bool bDirIn, uint64_t nCount, const char *szClient, const char *szPath)
+void transfer(char cLogType, uint64_t nCount, const char *szClient, const char *szPath)
 {
-	if(!fStat)
-		return;
 	lockguard g(&mx);
-
-	fprintf(fStat, acfg::verboselog ? "%lu|%c|%"PRIu64"|%s|%s\n" : "%lu|%c|%"PRIu64"\n",
-			(unsigned long) time(NULL), bDirIn ? 'I' : 'O', nCount,
-					szClient, szPath);
-
-	if(acfg::debug)
-		fflush(fStat);
+	if(!fStat.is_open())
+		return;
+	fStat << time(0) << '|' << cLogType << '|' << nCount;
+	if(acfg::verboselog)
+		fStat << '|' << szClient << '|' << szPath;
+	fStat << '\n';
+	if(acfg::debug) fStat.flush();
 }
 
 void misc(const string & sLine)
 {
-	if(!fStat)
-		return;
 	lockguard g(&mx);
+	if(!fStat.is_open())
+		return;
 
-	fprintf(fStat, "%lu|M|%s\n", (unsigned long) time(NULL), sLine.c_str());
+	fStat << time(0) << "|M|" << sLine << '\n';
 	
 	if(acfg::debug)
-		fflush(fStat);
+		fStat.flush();
 }
 
 /*
@@ -75,46 +83,40 @@ void misc(const string & sLine)
 
 void err(const char *msg, const char *client)
 {
-	if(!fErr)
-		return;
-	
 	lockguard g(&mx);
 
+	if(!fErr.is_open())
+		return;
+	
 	static char buf[32];
 	const time_t tm=time(NULL);
 	ctime_r(&tm, buf);
 	buf[24]=0;
-	if (client)
-		fprintf(fErr, "%s|%s: %s\n", buf, client, msg);
-	else
-		fprintf(fErr, "%s|%s\n", buf, msg);
+	fErr << buf << '|';
+	if(client)
+		fErr << client << ": ";
+	fErr << msg << '\n';
+
 #ifdef DEBUG
 	if(acfg::debug>=10)
-	{
-		fprintf(stderr, "%s|%s\n", buf, msg);
-		fflush(stderr);
-	}
+		cerr << buf << msg <<endl;
 #endif
-	fflush(fErr);
-	
+	fErr.flush();
 }
 
 void flush()
 {
 	lockguard g(&mx);
-	if(fErr)
-		fflush(fErr);
-	if(fStat)
-		fflush(fStat);
+	if(fErr.is_open()) fErr.flush();
+	if(fStat.is_open()) fStat.flush();
 }
 
-
 void close(bool bReopen)
 {
 	lockguard g(&mx);
-	if(acfg::debug>4) cerr << "Closing logs...\n";
-	checkForceFclose(fErr);
-	checkForceFclose(fStat);
+	if(acfg::debug>4) cerr << (bReopen ? "Reopening logs...\n" : "Closing logs...\n");
+	fErr.close();
+	fStat.close();
 	if(bReopen)
 		aclog::open();
 }
@@ -123,9 +125,9 @@ void close(bool bReopen)
 #define DAYSECONDS (3600*24)
 #define WEEKSECONDS (DAYSECONDS * 7)
 
-void GetStats(vector<tRowData> & out)
+void GetStats(deque<tRowData> & out)
 {
-	string sDataFile=acfg::cachedir+sPathSep+"_stats_dat";
+	string sDataFile=acfg::cachedir+SZPATHSEP"_stats_dat";
 
 	
 	out.clear();
@@ -151,10 +153,10 @@ void GetStats(vector<tRowData> & out)
 		
 #endif
 
-		tStrVec logs=ExpandFilePattern(acfg::logdir +
+		tStrDeq logs=ExpandFilePattern(acfg::logdir +
 				SZPATHSEP "apt-cacher*.log", false);
 
-		for (tStrVecIterConst it=logs.begin();it!=logs.end();it++)
+		for (tStrDeq::const_iterator it=logs.begin();it!=logs.end();it++)
 		{
 			if(acfg::debug)
 				cerr << "Reading log file: " << *it <<endl;
@@ -180,7 +182,7 @@ void GetStats(vector<tRowData> & out)
 				 if(when > out.front().to || when < out.back().from)
 					 continue;
 				 
-				 for(vector<tRowData>::reverse_iterator it=out.rbegin();
+				 for(deque<tRowData>::reverse_iterator it=out.rbegin();
 				 it!=out.rend(); 
 				 it++)
 				 {
@@ -200,10 +202,7 @@ void GetStats(vector<tRowData> & out)
 						 default:
 							 continue;
 						 }
-	  
 				 }
-				 
-				 
 			}
 		}
 		
@@ -256,9 +255,9 @@ string GetStatReport()
 {
 	string ret;
 	vector<char> buf(1024);
-	vector<aclog::tRowData> data;
+	deque<aclog::tRowData> data;
 	aclog::GetStats(data);
-	for (vector<aclog::tRowData>::iterator it = data.begin(); it != data.end(); it++)
+	for (deque<aclog::tRowData>::iterator it = data.begin(); it != data.end(); it++)
 	{
 
 		unsigned long reqMax = max(it->reqIn, it->reqOut);
@@ -326,13 +325,12 @@ errnoFmter::errnoFmter()
 {
 	char buf[32];
 	buf[0]=buf[31]=0x0;
-	msg=&buf[0];
 
 #ifdef _GNU_SOURCE
-#warning COMPILER BUG DETECTED -- _GNU_SOURCE was preset in C++ mode
-	msg=strerror_r(errno, buf, sizeof(buf)-1);
+//#warning COMPILER BUG DETECTED -- _GNU_SOURCE was preset in C++ mode
+	assign(strerror_r(errno, buf, sizeof(buf)-1));
 #else
-	strerror_r(errno, buf, sizeof(buf)-1);
+	assign(strerror_r(errno, buf, sizeof(buf)-1) ? "UNKNOWN ERROR" : buf);
 #endif
 }
 
@@ -367,9 +365,8 @@ t_logger::~t_logger()
 		stackDepths.erase(m_id);
 }
 
-MYSTD::ostringstream & t_logger::GetFmter()
+tSS & t_logger::GetFmter()
 {
-	m_strm.str("");
 	m_strm.clear();
 	for(UINT i=0;i<m_nLevel;i++)
 		m_strm << "\t";
@@ -381,11 +378,11 @@ void t_logger::Write(const char *pFile, unsigned int nLine)
 {
 	if(pFile)
 	{
-		const char *p=strrchr(pFile, cPathSep);
+		const char *p=strrchr(pFile, CPATHSEP);
 		pFile=p?(p+1):pFile;
 		m_strm << " [" << m_id << "@" << pFile << ":" << nLine <<"]";
 	}
-	aclog::err(m_strm.str().c_str());
+	aclog::err(m_strm.c_str());
 }
 
 #endif
