@@ -27,22 +27,30 @@ void check_algos();
 int main(int argc, char **argv) {
 
 	check_algos();
-	
-	signal(SIGTERM, term_handler);
-	signal(SIGINT, term_handler);
-	signal(SIGQUIT, term_handler);
-	signal(SIGUSR1, log_handler);
-	signal(SIGUSR2, SIG_IGN);
-	signal(SIGPIPE, SIG_IGN);
+	struct sigaction act;
+	memset(&act, 0, sizeof(act));
+
+	act.sa_handler = &term_handler;
+	sigaction(SIGTERM, &act, NULL);
+	sigaction(SIGINT, &act, NULL);
+	sigaction(SIGQUIT, &act, NULL);
+
+	act.sa_handler = &log_handler;
+	sigaction(SIGUSR1, &act, NULL);
+
+	act.sa_handler = SIG_IGN;
+	sigaction(SIGUSR2, &act, NULL);
+	sigaction(SIGPIPE, &act, NULL);
 #ifdef SIGIO
-	signal(SIGIO, SIG_IGN);
+	sigaction(SIGIO, &act, NULL);
 #endif
 #ifdef SIGXFSZ
-	signal(SIGXFSZ, SIG_IGN);
+	sigaction(SIGXFSZ, &act, NULL);
 #endif
 	
 	bool bForceCleanup(false);
-    for (int i=1; i<argc; i++) {
+    for (int i=1; i<argc; i++)
+    {
 
 		if (!strncmp(argv[i], "-h", 2))
 			usage();
@@ -58,12 +66,13 @@ int main(int argc, char **argv) {
 			bForceCleanup=true;
 			*(argv[i])=0x0; // ignore if ever checked
 		}
-		
     }
     
-    for (int i=1; i<argc; i++) {
+    for (int i=1; i<argc; i++)
+    {
     	
-        if(!strcmp(argv[i], "-c")) {
+        if(!strcmp(argv[i], "-c"))
+        {
         	i++;
         	if(i<argc)
         		acfg::ReadConfigDirectory(argv[i]);
@@ -80,7 +89,8 @@ int main(int argc, char **argv) {
 
 	}
     
-    if(!aclog::open()) {
+    if(!aclog::open())
+    {
         cerr << "Problem creating log files. Check permissions of the log directory, " << acfg::logdir<<endl;
         exit(1);
     }
@@ -98,20 +108,23 @@ int main(int argc, char **argv) {
 	if (acfg::foreground)
 		return conserver::Run();
 
-	if(daemon(0, 0))
-  {
-     cerr << "Failed to change to daemon mode" << endl;
-     exit(43);
-  }
+	if (daemon(0, 0))
+	{
+		cerr << "Failed to change to daemon mode" << endl;
+		exit(43);
+	}
 
-	if (!acfg::pidfile.empty()) {
+	if (!acfg::pidfile.empty())
+	{
 		mkbasedir(acfg::pidfile);
 		FILE *PID_FILE = fopen(acfg::pidfile.c_str(), "w");
-		if (PID_FILE != NULL) {
+		if (PID_FILE != NULL)
+		{
 			fprintf(PID_FILE, "%d", getpid());
 			checkForceFclose(PID_FILE);
 		}
 	}
+
 	return conserver::Run();
 	
 }
@@ -158,6 +171,8 @@ void log_handler(int)
 	aclog::close(true);
 }
 
+char *crashMe(NULL);
+
 void term_handler(int signum)
 {
 	switch (signum)
@@ -166,10 +181,13 @@ void term_handler(int signum)
 	case (SIGINT):
 	case (SIGQUIT):
 		aclog::close(false);
-		// enough of the other crap, too unreliable behaviour WRT C++ cleanup and workarounds in code
-		// would be lame. Just let the OS help with the suizide.
-		signal(SIGINT, SIG_DFL);
-		kill(getpid(), SIGINT);
+		// work is done, commit suicide
+		struct sigaction act;
+		memset(&act, 0, sizeof(act));
+		act.sa_handler = SIG_DFL;
+		if (sigaction(SIGINT, &act, NULL))
+			*crashMe = signum; // be sure not to end up in an infinite loop
+		raise(SIGINT);
 	default:
 		return;
 	}
@@ -210,34 +228,4 @@ void term_handler(int signum)
 		exit(0);
 	*/
 	
-
-// setup the signal handling as the every first thing
-	
-/*
- *     struct sigaction new_action, dummy_action;
-
-    new_action.sa_handler = termination_handler;
-    sigemptyset (&new_action.sa_mask);
-    new_action.sa_flags = 0;
-
-    //sigaction (SIGINT, NULL, &old_action);
-    //if (old_action.sa_handler != SIG_IGN)
-    sigaction (SIGINT, &new_action, NULL);
-    sigaction (SIGKILL, &new_action, NULL);
-    sigaction (SIGHUP, &new_action, NULL);
-    sigaction (SIGTERM, &new_action, NULL);
-
-    // FSCK LINUXTHREADS STOP KILLING MY APP EVEN IF LINKED STATICALY
-    dummy_action.sa_handler = SIG_IGN;
-    sigemptyset (&dummy_action.sa_mask);
-    dummy_action.sa_flags = 0;
-    sigaction(SIGRTMIN, &dummy_action, NULL);
-    sigaction(SIGRTMIN+1, &dummy_action, NULL);
-    sigaction(SIGRTMIN+2, &dummy_action, NULL);
-    sigaction(SIGRTMIN+3, &dummy_action, NULL);
-
-    int sigprocmask(int how, const sigset_t *set, sigset_t *oldset);
-    
-    // end signal setup
-*/
 #endif
