@@ -5,36 +5,47 @@
  *      Author: ed
  */
 
+#include <cstring>
+
 #include "bgtask.h"
 
 #include "lockable.h"
 #include "acfg.h"
 #include "meta.h"
 #include "filereader.h"
-#include "sockio.h"
 
 #include <limits.h>
 #include <errno.h>
 
+
+
 using namespace MYSTD;
 
 bool bSigTaskAbort=false;
 pthread_mutex_t abortMx=PTHREAD_MUTEX_INITIALIZER;
 
+
 time_t nMaintId(0), nLastId(0);
 lockable idLock;
 
-
-bgtask::bgtask(int fd) : maintenance(fd), m_repFileStream(NULL)
+bgtask::bgtask(int fd) : maintenance(fd)
 {
-
 }
 
 bgtask::~bgtask()
 {
-	checkForceFclose(m_repFileStream);
+
+	if(m_reportStream.is_open())
+		m_reportStream.close();
 }
 
+void _AddFooter(tSS &msg)
+{
+	msg << "<hr><address>Server " << acfg::agentname
+			<<" | <a href=\"https://flattr.com/thing/51105/Apt-Cacher-NG\">"
+			"Flattr it!</a> | <a href=\"http://www.unix-ag.uni-kl.de/~bloch/acng/\">"
+			"Apt-Cacher NG homepage</a></address>";
+}
 
 /*
  *  TODO: this is kept in expiration class for historical reasons. Should be moved to some shared upper
@@ -47,24 +58,24 @@ void bgtask::Run(const string &cmd)
 	{
 		lockguard g(&abortMx);
 		bSigTaskAbort=true;
-		const char hs[]="HTTP/1.1 302 Redirect\r\nLocation: ";
-		const char head[] = "\r\nConnection: close\r\n\r\n";
 		tStrPos nQuest=cmd.find("?");
-		if(nQuest==stmiss)
-			return; // whatever
-		SendRawData(hs, sizeof(hs)-1, MSG_MORE);
-		SendRawData(cmd.c_str(), nQuest, MSG_MORE);
-		SendRawData(head, sizeof(head)-1, MSG_MORE);
+		if(nQuest!=stmiss)
+		{
+			tSS buf(255);
+			buf << "HTTP/1.1 302 Redirect\r\nLocation: "
+				<< cmd.substr(0,nQuest)<< "\r\nConnection: close\r\n\r\n";
+			SendRawData(buf.rptr(), buf.size(), 0);
+		}
 		return;
 	}
 
 	SendChunkedPageHeader("200");
 
 	acbuf deco;
-	char *mark(NULL);
-	if(deco.initFromFile((acfg::confdir+sPathSep+m_szDecoFile).c_str()))
+	const char *mark(NULL);
+	if(deco.initFromFile((acfg::confdir+SZPATHSEP+m_szDecoFile).c_str()))
 	{
-		mark=strchr(deco.rptr(), '~');
+		mark=::strchr(deco.rptr(), '~');
 		if(mark)
 		{
 			SendChunk(deco.rptr(), mark-deco.rptr());
@@ -79,17 +90,19 @@ void bgtask::Run(const string &cmd)
 
 	static lockable onlyOneCanRun;
 
+	tSS msg;
+
 	if(!onlyOneCanRun.tryLock())
 	{
 		if(cmd.find("&readonly")==stmiss)
 		{
-			string msg("<font color=blue>A maintenance task is already running!</font><br>\n"
-					"<ul><li><a href=");
-			msg+=cmd+"&readonly#bottom";
-			msg+=">View current log messages</a></li><li><a href=/";
-			msg+=acfg::reportpage+">Return to main page</a></li></ul>"+
-			"<p>(<a href="+cmd+"&sigabort>Abort current task ASAP</a>)";
+			msg << "<font color=blue>A maintenance task is already running!</font><br>\n"
+					"<ul><li><a href=" <<  cmd << "&readonly#bottom>"
+							"View current log messages</a></li><li><a href=/"
+					<<acfg::reportpage << ">Return to main page</a></li></ul>"
+					"<p>(<a href=" << cmd << "&sigabort>Abort current task ASAP</a>)";
 
+			_AddFooter(msg);
 			SendChunk(msg);
 		}
 		else
@@ -100,8 +113,9 @@ void bgtask::Run(const string &cmd)
 				id=nMaintId;
 			}
 			DumpLog(id);
-			string msg("<br>\n<a name=bottom>To be continued... (reload this page to see recent output)<br><a href=/");
-			msg	+=acfg::reportpage+">Return to main page</a>\n";
+			msg << "<br>\n<a name=bottom>To be continued... (reload this page to see recent output)"
+					"<br><a href=/" << acfg::reportpage << ">Return to main page</a>\n";
+			_AddFooter(msg);
 			SendChunk(msg);
 
 		}
@@ -111,12 +125,16 @@ void bgtask::Run(const string &cmd)
 		// locking worked but not intended to act now
 		if(cmd.find("&readonly")!=stmiss)
 		{
-			string msg("<font color=blue>Previous maintainenance task is completed!</font><br>\n"
-						"<ul><li><a href=/");
-			msg	+=acfg::reportpage+">Return to main page</a></li></ul><p>Last log contents:<br>\n";
+			msg << "<font color=blue>Previous maintainenance task is completed!</font><br>\n"
+						"<ul><li><a href=/" << acfg::reportpage
+						<< ">Return to main page</a></li></ul><p>Last log contents:<br>\n";
 			SendChunk(msg);
 
 			DumpLog(nLastId);
+
+			msg.clear();
+			_AddFooter(msg);
+			SendChunk(msg);
 		}
 		else
 		{
@@ -127,11 +145,9 @@ void bgtask::Run(const string &cmd)
 				lockguard g(idLock);
 				nMaintId=time(NULL);
 			}
-
-			char buf[PATH_MAX];
-			snprintf(buf, sizeof(buf), "%s"SZPATHSEP"maint_%lu.log",
-					acfg::logdir.c_str(), nMaintId);
-			m_repFileStream=fopen(buf, "w");
+			tSS path(acfg::logdir.length()+24);
+			path<<acfg::logdir<<CPATHSEP<<"maint_" << nMaintId << ".log";
+			m_reportStream.open(path.c_str(), ios::out);
 
 			{
 				lockguard g(&abortMx);
@@ -147,8 +163,10 @@ void bgtask::Run(const string &cmd)
 				bSigTaskAbort=false;
 			}
 
-			SendChunk(string("<br>\n<a href=\"/")+acfg::reportpage
-					+"\">Return to main page</a>");
+			msg.clear();
+			msg<<"<br>\n<a href=\"/"<<acfg::reportpage<<"\">Return to main page</a>";
+			_AddFooter(msg);
+			SendChunk(msg);
 
 			{
 				lockguard g(idLock);
@@ -156,11 +174,6 @@ void bgtask::Run(const string &cmd)
 				nMaintId=0;
 			}
 
-			if(m_repFileStream)
-			{
-				fclose(m_repFileStream);
-				m_repFileStream=0;
-			}
 		}
 		onlyOneCanRun.unlock();
 	}
@@ -182,27 +195,22 @@ void bgtask::DumpLog(time_t id)
 	filereader reader;
 
 	if (id<=0)
-		goto log_na;
+		return;
 
-	char buf[PATH_MAX];
-	snprintf(buf, sizeof(buf), "%s"SZPATHSEP"maint_%lu.log",
-			acfg::logdir.c_str(), id);
-	if (!reader.OpenFile(buf))
-		goto log_na;
-
-	maintenance::SendChunk(reader.GetBuffer(), reader.GetSize());
-	return;
-
-	log_na:
-	SendChunk(string("Log not available"));
+	tSS path(acfg::logdir.length()+24);
+	path<<acfg::logdir<<CPATHSEP<<"maint_" << id << ".log";
+	if (!reader.OpenFile(path))
+		maintenance::SendChunk(_SZ2PS("Log not available"));
+	else
+		maintenance::SendChunk(reader.GetBuffer(), reader.GetSize());
 }
 
 void bgtask::SendChunk(const char *data, size_t len)
 {
-	if (m_repFileStream)
+	if(m_reportStream.is_open())
 	{
-		fwrite(data, sizeof(char), len, m_repFileStream);
-		fflush(m_repFileStream);
+		m_reportStream.write(data, len);
+		m_reportStream.flush();
 	}
 	maintenance::SendChunk(data, len);
 }
