@@ -43,9 +43,9 @@ bool CAddrInfo::Resolve(const string & sHostname, const string &sPort,
 
 	if (0!=r)
 	{
-		sErrorBuf=string("503 DNS error for hostname ")+sHostname+": "
-				+gai_strerror(r)+". If "+sHostname
-				+" refers to a configured cache repository, please check the corresponding configuration file.";
+		sErrorBuf=(tSS(250)<<"503 DNS error for hostname "<<sHostname<<": "<<gai_strerror(r)
+				<<". If "<<sHostname<<" refers to a configured cache repository, "
+				"please check the corresponding configuration file.");
 		return false;
 	}
 
@@ -110,7 +110,7 @@ CAddrInfo::SPtr CAddrInfo::CachedResolve(const string & sHostname, const string
 	else // error, return empty ptr
 	{
 		// better log here, often APT does not display stupid user errors
-		aclog::err(string("Error resolving ")+sHostname+": " +sErrorMsgBuf);
+		aclog::err( (tSS()<<"Error resolving "<<sHostname<<": " <<sErrorMsgBuf).c_str());
 		p.reset();
 	}
 
