@@ -83,26 +83,32 @@ void * ThreadAction(void *)
 }
 
 // Thread creation helper, global stuff must be protected while using this
-bool SpawnThread()
+inline bool _SpawnThreadWhenNeeded()
 {
-	//USRDBG(3, "Creating connection worker");
-	
-	 // artificial limit hit?
-	if(g_nAllConThreadCount>=acfg::tpthreadmax)
-		return false;
-	
-	g_nAllConThreadCount++;
-	
-	pthread_t thr;
-	pthread_attr_t attr; // be detached from the beginning 
-	pthread_attr_init(&attr);
-	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-
-	if(0==pthread_create(&thr, NULL, ThreadAction, NULL))
+	if(g_nStandbyThreads>0)
 		return true;
+
+	// check the kill-switch
+	if(g_nAllConThreadCount+1>=acfg::tpthreadmax)
+		return false;
+
+	pthread_t thr;
+	pthread_attr_t attr; // be detached from the beginning
+	bool bRet(false);
 	
-	g_nAllConThreadCount--;
-	return false;
+	if(pthread_attr_init(&attr))
+		goto thr_deinit_attr;
+
+	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
+	if(0 == pthread_create(&thr, &attr, ThreadAction, NULL))
+	{
+		g_nAllConThreadCount++;
+		bRet=true;
+	}
+
+	thr_deinit_attr:
+	pthread_attr_destroy(&attr);
+	return bRet;
 }
 
 void SetupConAndGo(int fd, const char *szClientName=NULL)
@@ -143,7 +149,7 @@ void SetupConAndGo(int fd, const char *szClientName=NULL)
 		goto failure_mode;
 	}
 
-	if (g_nStandbyThreads==0 && !SpawnThread())
+	if (!_SpawnThreadWhenNeeded())
 	    goto failure_mode;
 	
 	g_freshConQueue.push_back(c);
