@@ -99,7 +99,7 @@ void tDlJob::BlacklistBackend()
 }
 
 // needs connectedHost, blacklist, output buffer from the parent, proxy mode?
-bool tDlJob::AppendRequest(string sForThisHostOnly, string &head)
+bool tDlJob::AppendRequest(string sForThisHostOnly, tSS &head)
 {
 	LOGSTART2("tDlJob::AppendRequest", "hostfilter: " << sForThisHostOnly);
 	if(! m_pStorage)
@@ -120,14 +120,13 @@ bool tDlJob::AppendRequest(string sForThisHostOnly, string &head)
 		return false;
 	}
 
-	head += m_sRequestCmd + (m_parent->m_proxy ? RemoteUri() : RemotePath()) + " HTTP/1.1\r\n"
-	"Host: " + GetPeerName() + "\r\n"
-	"Connection: keep-alive\r\n";
+	head << m_sRequestCmd << (m_parent->m_proxy ? RemoteUri() : RemotePath()) << " HTTP/1.1\r\n"
+	"Host: " << GetPeerName() << "\r\nConnection: keep-alive\r\n";
 	
 	if (m_parent->m_proxy) // also add authorization if there is any
 	{
 		ldbg("using proxy");
-		head+=m_parent->m_proxy->sPath+"Proxy-Connection: keep-alive\r\n";
+		head << m_parent->m_proxy->sPath << "Proxy-Connection: keep-alive\r\n";
 	}
 
 
@@ -136,8 +135,8 @@ bool tDlJob::AppendRequest(string sForThisHostOnly, string &head)
 		bool bSetRange(false), bSetIfRange(false);
 		
 		lockguard g(m_pStorage.get());
-		const header *pHead = m_pStorage->GetHeaderUnlocked();
-		const char *p=pHead->h[header::LAST_MODIFIED];
+		const header &pHead = m_pStorage->GetHeaderUnlocked();
+		const char *p=pHead.h[header::LAST_MODIFIED];
 		
 		if (m_pStorage->m_bCheckFreshness)
 		{
@@ -158,33 +157,24 @@ bool tDlJob::AppendRequest(string sForThisHostOnly, string &head)
 			bSetRange=true;
 		}
 
+        /* use APT's trick - set the starting position one byte lower -
+         * this way the server has to send at least one byte if the assumed
+         * position is correct, and we never get a 416 error (one byte
+         * waste is acceptable).
+         * */
 		if (bSetRange)
-		{
-            /* use APT's trick - set the starting position one byte lower -
-             * this way the server has to send at least one byte if the assumed
-             * position is correct, and we never get a 416 error (one byte
-             * waste is acceptable). 
-             * */
-			char buf[50];
-			sprintf(buf, "Range: bytes=" OFF_T_FMT 
-                    "-\r\n", m_pStorage->m_nSizeSeen - 1 );
-			head+=buf;
-		}
+			head << "Range: bytes=" << (m_pStorage->m_nSizeSeen - 1) << "-\r\n";
+
 		if (bSetIfRange)
-		{
-			head += "If-Range: ";
-			head += p;
-			head += "\r\n";
-		}
+			head << "If-Range: " << p << "\r\n";
 	}
-	// Debian Apt-Cacher/" ACVERSION 
-	head+="User-Agent: ";
-	head += acfg::agentname + "\r\n";
 	
-	if(acfg::exporigin)
-		(head+="X-Forwarded-For: ")+=m_parent->m_sXForwardedFor+"\r\n";
+	head << acfg::agentheader;
+
+	if(acfg::exporigin && !m_parent->m_sXForwardedFor.empty())
+		head << "X-Forwarded-For: " << m_parent->m_sXForwardedFor << "\r\n";
 	
-	head += "\r\n";
+	head << "\r\n";
 			
 	ldbg("Request cooked: " << head);
 	
@@ -305,7 +295,7 @@ tDlJob::tDlResult tDlJob::ProcessIncomming(acbuf & inBuf, string & sErrorRet)
 
         h.set(header::XORIG, RemoteUri());
 
-        if ( ! m_pStorage->DownloadStartedStoreHeader(h))
+        if ( ! m_pStorage->DownloadStartedStoreHeader(h, inBuf.rptr()))
         {
             ldbg("Item dl'ed by others or in error state --> drop it, reconnect");
             m_DlState=STATE_GETDATA; // XXX maybe introduce a new error state, this is a kludge
@@ -334,8 +324,8 @@ tDlJob::tDlResult tDlJob::ProcessIncomming(acbuf & inBuf, string & sErrorRet)
 	case (STATE_GETCHUNKHEAD):
 	{
 		ldbg("STATE_GETCHUNKHEAD");
-		char *p=inBuf.c_str();
-		char *e=strstr(p, "\r\n");
+		const char *p=inBuf.c_str();
+		const char *e=strstr(p, "\r\n");
 		if(e==p)
 		{ // came back from reading, drop remaining junk? 
 			inBuf.drop(2);
