@@ -13,9 +13,17 @@
 
 using namespace MYSTD;
 
-header const * fileitem::GetHeaderUnlocked()
+header const & fileitem::GetHeaderUnlocked()
 {
-	return &m_head;
+	return m_head;
+}
+
+string fileitem::GetHttpMsg()
+{
+	setLockGuard;
+	if(m_head.frontLine.length()>9)
+		return m_head.frontLine.substr(9);
+	return m_head.frontLine;
 }
 
 fileitem::fileitem(MYSTD::string sPath) :
@@ -23,7 +31,7 @@ fileitem::fileitem(MYSTD::string sPath) :
 	m_bCheckFreshness(true),
 	m_nIncommingCount(0),
 	m_nSizeSeen(0),
-	m_sPath(acfg::cachedir+sPathSep+sPath),
+	m_sPath(acfg::cachedir+SZPATHSEP+sPath),
 	m_sKey(sPath),
 	m_nSizeChecked(0),
 	m_filefd(-1),
@@ -202,10 +210,10 @@ FiStatus fileitem::Setup(bool bCheckFreshness, bool bIgnoreCached) {
 inline void _LogWithErrno(const char *msg, const string & sFile)
 {
 	aclog::errnoFmter f;
-	aclog::err( (sFile+" storage error ["+msg+"]: "+f.msg).c_str());
+	aclog::err( (sFile+" storage error ["+msg+"]: "+f).c_str());
 }
 
-bool fileitem::DownloadStartedStoreHeader(const header & h)
+bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextData)
 {
 	LOGSTART("fileitem::DownloadStartedStoreHeader");
 
@@ -227,18 +235,18 @@ bool fileitem::DownloadStartedStoreHeader(const header & h)
 	// status will change, most likely... ie. BOUNCE action
 	notifyAll();
 
-#define SETERROR(x) { m_head.frontLine="HTTP/1.1 "; m_head.frontLine+=x; status=FIST_ERROR; _LogWithErrno(x, m_sPath); }
+#define SETERROR(x) { m_bAllowStoreData=false; m_head.frontLine=(tSS() << "HTTP/1.1 " << (x)); status=FIST_ERROR; _LogWithErrno(x, m_sPath); }
 #define SETERRORKILLFILE(x) { SETERROR(x); goto kill_file; }
-#define BOUNCE(x) { SETERROR(x); _LogWithErrno(x, m_sPath); return false; }
+#define BOUNCE(x) { SETERROR(x); return false; }
+
+	m_head.set(header::XORIG, h.h[header::XORIG]);
 	
 	int code=h.getStatus();
 	ldbg("http code: "<< code);
-	bool bStoreData(false);
 	string sHeadPath=m_sPath+".head";
 	
 	if(200 == code)
 	{
-		bStoreData=m_bAllowStoreData;
 		m_nSizeChecked=0;
 		m_head=h;
 		const char *p=h.h[header::CONTENT_LENGTH];
@@ -273,7 +281,6 @@ bool fileitem::DownloadStartedStoreHeader(const header & h)
 			BOUNCE("500 Server reports illegal range");
 	
 		m_nSizeChecked=myfrom;
-		bStoreData=m_bAllowStoreData;
 		
 		hint_start=myfrom;
 		hint_length=mylen;
@@ -282,32 +289,53 @@ bool fileitem::DownloadStartedStoreHeader(const header & h)
 		m_head.frontLine="HTTP/1.1 200 OK";
 		m_head.del(header::CONTENT_RANGE);
 		m_head.set(header::CONTENT_LENGTH, mylen);
-		m_head.set(header::XORIG, h.h[header::XORIG]);
+
+		// special optimization... if "-1 trick" was used, don't reopen that file for writing later
+		if(m_bCheckFreshness && m_nSizeSeen == mylen && m_nSizeChecked == mylen-1)
+		{
+			int fd=open(m_sPath.c_str(), O_RDONLY);
+			if(fd>=0)
+			{
+				if(m_nSizeChecked==lseek(fd, m_nSizeChecked, SEEK_SET))
+				{
+					char c;
+					if(1 == read(fd, &c, 1) && pNextData && c == *pNextData)
+					{
+						aclog::err("known data hit, don't write to...");
+						aclog::err(m_sPath);
+						m_bAllowStoreData=false;
+						m_nSizeChecked=mylen;
+					}
+				}
+				forceclose(fd);
+			}
+		}
 	}
-	else if(416 == code) // that's always bad; it cannot be complete (-1 trick) -> kill cached file ASAP
+	else if(416 == code)
 	{
+		// that's always bad; it cannot have been complete before (the -1 trick)
+		// -> kill cached file ASAP
+		m_bAllowStoreData=false;
 		SETERRORKILLFILE("503 Server disagrees on file size, cleaning up");
 	}
 	else
 	{
-		bStoreData=false;
-		// have a clear one with just the error message
+		m_bAllowStoreData=false;
+		// have a clean header with just the error message
 		m_head.frontLine=h.frontLine;
 		m_head.type=header::ANSWER;
-		m_head.set(header::XORIG, h.h[header::XORIG]);
-
-		// pass-through the new location if needed
-		if(code == 301 || code == 302)
+		if(h.h[header::LOCATION] && (301==code||302==code||303==code||307==code))
 			m_head.set(header::LOCATION, h.h[header::LOCATION]);
 	}
-			
-	if(bStoreData)
+
+	if(acfg::debug>1)
+		aclog::misc(string("Download of ")+m_sKey+" started");
+
+	if(m_bAllowStoreData)
 	{
+
 		// using adaptive Delete-Or-Replace-Or-CopyOnWrite strategy
 		
-		if(acfg::debug>1)
-			aclog::misc(string("Download of ")+m_sKey+" started");
-		
 		// First opening the file first to be sure that it can be written. Header storage is the critical point,
 		// every error after that leads to full cleanup to not risk inconsistent file contents 
 		
@@ -371,8 +399,10 @@ bool fileitem::DownloadStartedStoreHeader(const header & h)
 		
 		// unlink and retry
 		if(count<0)
+		{
 			unlink(sHeadPath.c_str());
-		count=m_head.StoreToFile(sHeadPath);
+			count=m_head.StoreToFile(sHeadPath);
+		}
 
 		if(count<0)
 			SETERRORKILLFILE( (-count!=ENOSPC) ? "503 Cache storage error" : "503 OUT OF DISK SPACE");
@@ -390,7 +420,13 @@ bool fileitem::DownloadStartedStoreHeader(const header & h)
 
 	kill_file:
 	if(m_filefd>=0)
+	{
+#if _POSIX_SYNCHRONIZED_IO > 0
+		fsync(m_filefd);
+#endif
 		forceclose(m_filefd);
+	}
+
 	unlink(m_sPath.c_str());
 	unlink(sHeadPath.c_str());
 	
@@ -415,7 +451,7 @@ bool fileitem::StoreFileData(const char *data, unsigned int size) {
 		status=FIST_COMPLETE;
 
 		if(acfg::debug>1)
-			aclog::misc(string("Download of ")+m_sKey+" finished");
+			aclog::misc(tSS()<<"Download of "<<m_sKey<<" finished");
 		
 		// we are done! Fix header from chunked transfers?
 		if (m_filefd>=0 && ! m_head.h[header::CONTENT_LENGTH])
@@ -428,6 +464,9 @@ bool fileitem::StoreFileData(const char *data, unsigned int size) {
 	{
 		status = FIST_DLRECEIVING;
 
+		if (!m_bAllowStoreData)
+			return true;
+
 		if (m_filefd>=0)
 		{
 			while(size>0)
