@@ -13,7 +13,9 @@ using namespace MYSTD;
 #include "fileitem.h"
 #include "dlcon.h"
 #include "sockio.h"
-
+#include "fileio.h" // for ::stat
+#include <dirent.h>
+#include <algorithm>
 #include "maintenance.h"
 
 #ifdef HAVE_LINUX_SENDFILE
@@ -24,8 +26,15 @@ using namespace MYSTD;
 
 #define REPLEVEL (m_type==rechecks::FILE_INDEX ? 4 : (m_type==rechecks::FILE_PKG ? 5 : SPAMLEVEL))
 
-//void DispatchAndRunMaintTask(const MYSTD::string &, int, const MYSTD::string &);
 
+#define SPECIAL_FD -42
+inline bool IsValidFD(int fd) { return fd>=0 || SPECIAL_FD == fd; }
+
+/*
+ * Unlike the regular store-and-forward file item handler, this ones does not store anything to
+ * harddisk. Instead, it uses the download buffer and lets job object send the data straight from
+ * it to the client.
+ */
 class tPassThroughFitem : public fileitem
 {
 protected:
@@ -44,7 +53,7 @@ public:
 		return status = FIST_INITED;
 	}
 	virtual void Unreg() {}; // NOOP, since this item is not shared
-	virtual int GetFileFd() { return 1; }; // something, don't care
+	virtual int GetFileFd() { return SPECIAL_FD; }; // something, don't care for now
 	virtual bool StoreFileData(const char *data, unsigned int size)
 	{
 		setLockGuard;
@@ -58,12 +67,7 @@ public:
 			return false;
 
 		if (size == 0)
-		{
 			status = FIST_COMPLETE;
-			// we are done! Fix header from chunked transfers?
-			if (m_filefd >= 0 && !m_head.h[header::CONTENT_LENGTH])
-				m_head.set(header::CONTENT_LENGTH, m_nSizeChecked);
-		}
 		else
 		{
 			status = FIST_DLRECEIVING;
@@ -87,11 +91,18 @@ public:
 	ssize_t SendData(int out_fd, int, off_t &nSendPos, size_t nMax2SendNow)
 	{
 		setLockGuard;
-		while(0 == m_nConsumable && status<FIST_ERROR)
+
+		while(0 == m_nConsumable && status<FIST_ERROR
+				&& ! (m_nSizeChecked==0 && status==FIST_COMPLETE))
+		{
 			wait();
+		}
 		if (status >= FIST_ERROR || !m_pData)
 			return -1;
 
+		if(!m_nSizeChecked)
+			return 0;
+
 		ssize_t r = write(out_fd, m_pData, min(nMax2SendNow, m_nConsumable));
 		if (r < 0 && (errno == EAGAIN || errno == EINTR)) // harmless
 			r = 0;
@@ -111,6 +122,38 @@ public:
 	}
 };
 
+// fileitem replacement with custom storage for local data
+class tGeneratedFitemBase : public fileitem
+{
+public:
+	virtual void Unreg() {}; // NOOP, since this item is not shared
+	virtual int GetFileFd() { return SPECIAL_FD; }; // something, don't care for now
+	tSS m_data;
+
+	tGeneratedFitemBase(const string &sFitemId, const char *szFrontLineMsg) : fileitem(sFitemId), m_data(256)
+	{
+		status=FIST_COMPLETE;
+		m_head.type = header::ANSWER;
+		m_head.frontLine = "HTTP/1.1 ";
+		m_head.frontLine += (szFrontLineMsg ? szFrontLineMsg : "500 Internal Failure");
+		m_head.set(header::CONTENT_TYPE, _SZ2PS("text/html") );
+	}
+	ssize_t SendData(int out_fd, int, off_t &nSendPos, size_t nMax2SendNow)
+	{
+		if (status >= FIST_ERROR || out_fd<0)
+			return -1;
+		int r = m_data.syswrite(out_fd, nMax2SendNow);
+		if(r>0) nSendPos+=r;
+		return r;
+	}
+	inline void seal()
+	{
+		// seal the item
+		m_nSizeChecked = m_data.size();
+		m_head.set(header::CONTENT_LENGTH, m_nSizeChecked);
+	}
+};
+
 ssize_t sendfile_generic(int out_fd, int in_fd, off_t *offset, size_t count)
 {
 	LOGSTART("sendfile_generic (fallback)");
@@ -178,37 +221,50 @@ job::job(header *h, con *pParent) :
 	m_bChunkMode(false),
 	//m_bCloseCon(false),
 	m_bIsHttp11(false),
-	m_state(STATE_FRESH),
+	m_state(STATE_SEND_MAIN_HEAD),
 	m_pReqHead(h),
 	m_nSendPos(0),
 	m_nRangeLast(OFFT_MAX-1), // range size should never overflow
-	m_nChunkRemainingBytes(0)
+	m_nAllDataCount(0),
+	m_nChunkRemainingBytes(0),
+	m_type(rechecks::FILE_INVALID)
 {
-	LOGSTART2("job::job", "job creating, " << m_pReqHead->frontLine << " and this: " << this);
-	m_nAllDataCount=0;
+	LOGSTART2("job::job", "job creating, " << m_pReqHead->frontLine << " and this: " << uintptr_t(this));
 }
 
+string miscError("(HTTP error page)");
+
 job::~job()
 {
 	LOGSTART("job::~job");
-	m_pParentCon->LogDataCounts(m_sFileLoc,
+
+	bool bErr=m_sFileLoc.empty();
+	m_pParentCon->LogDataCounts(
+			( bErr ? (m_pItem ? m_pItem->GetHttpMsg() : miscError ) : m_sFileLoc ),
 			m_pReqHead->h[header::XFORWARDEDFOR],
 			(m_pItem ? m_pItem->GetTransferCount() : 0),
-			m_nAllDataCount);
+			m_nAllDataCount, bErr);
 	
 	checkforceclose(m_filefd);
 
 	if (m_pItem)
-	{
 		m_pItem->Unreg();
-		m_pItem.reset();
-	}
-	
-	if(m_pReqHead)
+
+	delete m_pReqHead;
+}
+
+const string & GetMimeType(const string &path)
+{
+	tStrPos dpos = path.find_last_of('.');
+	if (dpos != stmiss)
 	{
-		delete m_pReqHead;
-		m_pReqHead=NULL;
+		NoCaseStringMap::const_iterator it = acfg::mimemap.find(path.substr(
+				dpos + 1));
+		if (it != acfg::mimemap.end())
+			return it->second;
 	}
+	static string nix("");
+	return nix;
 }
 
 void replaceChars(string &s, const char *szBadChars, char goodChar)
@@ -222,10 +278,184 @@ void replaceChars(string &s, const char *szBadChars, char goodChar)
 			}
 }
 
+inline void job::HandleLocalDownload(const string &visPath,
+		const string &fsBase, const string &fsSubpath)
+{
+	struct stat stbuf;
+	string absPath = fsBase+SZPATHSEP+fsSubpath;
+	if (0!=::stat(absPath.c_str(), &stbuf))
+	{
+		switch(errno)
+		{
+		case EACCES:
+			SetErrorResponse("403 Permission denied");
+			break;
+		case EBADF:
+		case EFAULT:
+		case ENOMEM:
+		case EOVERFLOW:
+		default:
+			//aclog::err("Internal error");
+			SetErrorResponse("500 Internal server error");
+			break;
+		case ELOOP:
+			SetErrorResponse("500 Infinite link recursion");
+			break;
+		case ENAMETOOLONG:
+			SetErrorResponse("500 File name too long");
+			break;
+		case ENOENT:
+		case ENOTDIR:
+			SetErrorResponse("404 File or directory not found");
+			break;
+		}
+		return;
+	}
+/*
+	// simplified version, just converts a string into a page
+	class bufferitem : public tPassThroughFitem, private string
+	{
+	public:
+		bufferitem(const string &sId, const string &sData)
+		: tPassThroughFitem(sId), string(sData)
+		{
+			status = FIST_COMPLETE;
+			m_pData=c_str();
+			m_nConsumable=length();
+			m_nSizeChecked=m_nConsumable;
+			m_head.frontLine.assign(_SZ2PS("HTTP/1.1 200 OK"));
+			m_head.set(header::CONTENT_LENGTH, length());
+			m_head.set(header::CONTENT_TYPE, _SZ2PS("text/html") );
+			m_head.type=header::ANSWER;
+		}
+	};
+*/
+
+	if(S_ISDIR(stbuf.st_mode))
+	{
+		// unconfuse the browser
+		if(!endsWithSzAr(visPath, SZPATHSEPUNIX))
+		{
+			class dirredirect : public tGeneratedFitemBase
+			{
+			public:	dirredirect(const string &visPath)
+			: tGeneratedFitemBase(visPath, "301 Moved Permanently")
+				{
+					m_head.set(header::LOCATION, visPath+"/");
+					m_data << "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">"
+					"<html><head><title>301 Moved Permanently</title></head><body><h1>Moved Permanently</h1>"
+					"<p>The document has moved <a href=\""+visPath+"/\">here</a>.</p></body></html>";
+
+					seal();
+				}
+			};
+			m_pItem.reset(static_cast<fileitem*>(new dirredirect(visPath)));
+			return;
+		}
+
+		class listing: public tGeneratedFitemBase
+		{
+		public:
+			listing(const string &visPath) :
+				tGeneratedFitemBase(visPath, "200 OK")
+			{
+				seal(); // for now...
+			}
+		};
+		listing *p=new listing(visPath);
+		m_pItem.reset(static_cast<fileitem*>(p)); // assign to smart pointer ASAP, operations might throw
+		tSS & page = p->m_data;
+
+		page << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">"
+		"<html><head><title>Index of " << visPath << "</title></head>"
+		"<body><h1>Index of " << visPath << "</h1>"
+		"<table><tr><th>&nbsp;</th><th>Name</th><th>Last modified</th><th>Size</th></tr>"
+		"<tr><th colspan=\"4\"><hr></th></tr>";
+
+		DIR *dir = opendir(absPath.c_str());
+		if (!dir) // weird, whatever... ignore...
+			page<<"ERROR READING DIRECTORY";
+		else
+		{
+			// quick hack, good enough
+			MYSTD::map<string, tSS> sortMap;
+			for(struct dirent dp, *pEndTest;
+					0==readdir_r(dir, &dp, &pEndTest) && pEndTest; )
+			{
+				if (0!=::stat((absPath+SZPATHSEP+dp.d_name).c_str(), &stbuf))
+					continue;
+
+				bool bDir=S_ISDIR(stbuf.st_mode);
+
+				char datestr[32]={0};
+				struct tm tmtimebuf;
+				strftime(datestr, sizeof(datestr)-1,
+						"%d-%b-%Y %H:%M", localtime_r(&stbuf.st_mtime, &tmtimebuf));
+
+				tSS &line = sortMap[string(bDir?"a":"b")+dp.d_name];
+				if(bDir)
+					line << "[DIR]";
+				else if(startsWithSz(GetMimeType(dp.d_name), "image/"))
+					line << "[IMG]";
+				else
+					line << "[&nbsp;&nbsp;&nbsp;]";
+				line <<  "</td><td><a href=\"" << dp.d_name <<	+(bDir? "/\">" : "\">" ) << dp.d_name
+						<< "</a></td><td>" << datestr << "</td><td align=\"right\">";
+				if(bDir)
+					line << "-";
+				else
+					line << stbuf.st_size;
+			}
+			closedir(dir);
+			for(MYSTD::map<string, tSS>::const_iterator it=sortMap.begin(); it!=sortMap.end(); it++)
+				page << "<tr><td valign=\"top\">" << it->second << "</td></tr>\r\n";
+		}
+		page << "<tr><td colspan=\"4\">";
+		_AddFooter(page);
+		page << "</td></tr></table></body></html>";
+		p->seal();
+		return;
+	}
+	if(!S_ISREG(stbuf.st_mode))
+	{
+		SetErrorResponse("403 Unsupported data type");
+		return;
+	}
+	/*
+	 * This variant of file item handler sends a local file. The
+	 * header data is generated as needed.
+	 */
+	class tLocalGetFitem : public fileitem
+	{
+	public:
+		tLocalGetFitem(string s, string sLocalPath, struct stat &stdata) : fileitem(s)
+		{
+			m_bAllowStoreData=false;
+			m_sPath=sLocalPath;
+			status=FIST_COMPLETE;
+			m_nSizeChecked=m_nSizeSeen=stdata.st_size;
+			m_bCheckFreshness=false;
+			m_head.type=header::ANSWER;
+			m_head.frontLine="HTTP/1.1 200 OK";
+			m_head.set(header::CONTENT_LENGTH, stdata.st_size);
+			const string &sMimeType=GetMimeType(sLocalPath);
+			if(!sMimeType.empty())
+				m_head.set(header::CONTENT_TYPE, sMimeType);
+		};
+		virtual void Unreg() {}; // NOOP, since this item is not shared
+	};
+	m_pItem.reset(static_cast<fileitem*>(new tLocalGetFitem(visPath, absPath, stbuf)));
+}
+
 void job::PrepareDownload() {
 
     LOGSTART("job::PrepareDownload");
     
+#ifdef DEBUG
+    acfg::localdirs["stuff"]="/tmp/stuff";
+    aclog::err(m_pReqHead->ToString());
+#endif
+
     string sRawUriPath, sPathResdiual;
     tHttpUrl tUrl; // parsed URL
         
@@ -235,42 +465,32 @@ void job::PrepareDownload() {
     FiStatus fistate(FIST_FRESH);
     bool bPtMode(false);
     bool bForceFreshnessChecks(false); // force update of the file, i.e. on dynamic index files?
-
-    if(!m_sErrorMsgBuffer.empty())
-       	return ;
+	tStrVec tokens;
     
     if(m_pReqHead->type!=header::GET && m_pReqHead->type!=header::HEAD)
     	goto report_invpath;
     
-    m_bIsHttp11=(m_pReqHead->frontLine.length()>8 && 
-    		0 == m_pReqHead->frontLine.compare(
-					m_pReqHead->frontLine.length()-8, 8, "HTTP/1.1"));
-    
+	if (3 != Tokenize(m_pReqHead->frontLine, SPACECHARS, tokens))
+		goto report_invpath;
 
-	{
-		/*
-		// persistent connection?
-	    const char *p=m_pReqHead->h[header::CONNECTION];
-		if (!p) 
-			m_bCloseCon = true; // enough! too many stupid clients out there; previous default: "close" IFF not http 1.1 !m_bIsHttp11; 
-		else // normalize case
-			m_bCloseCon = (0 == strcasecmp(p, "close"));
-		*/
-		
-		tStrVec tmp;
-		if(3 == Tokenize(m_pReqHead->frontLine, SPACECHARS, tmp))
-			sRawUriPath=tmp[1];
-		else
-			goto report_invpath; // invalid uri
-	}
+	sRawUriPath = tokens[1];
+	USRDBG(6, "Raw request URI: " << sRawUriPath);
 
+	m_bIsHttp11=(tokens[2] == "HTTP/1.1");
+	// a sane default? close for http 1.0, keep for 1.1.
+	// But if set by the client, comply!
+	m_bClientWants2Close =!m_bIsHttp11;
+	if(m_pReqHead && m_pReqHead->h[header::CONNECTION])
+		m_bClientWants2Close = 0==strncasecmp(m_pReqHead->h[header::CONNECTION], "close", 5);
+
+    // "clever" file system browsing attempt?
+	if(stmiss != sRawUriPath.find("..")
+			|| stmiss != sRawUriPath.find("/_actmp")
+			|| startsWithSz(sRawUriPath, "/_"))
+		goto report_notallowed;
 
     MYTRY
 	{
-    	LOG("raw uri: " << sRawUriPath);
-    	// filesystem browsing attempt?
-		if(stmiss != sRawUriPath.find("..") || stmiss != sRawUriPath.find("/_actmp"))
-			goto report_notallowed;
 
 		if (0==sRawUriPath.compare(0, 12, "apt-cacher?/"))
 		sRawUriPath.erase(0, 12);
@@ -319,6 +539,18 @@ void job::PrepareDownload() {
 				return;
 			}
 		}
+
+		for(tStringMap::const_iterator it=acfg::localdirs.find(tUrl.sHost);
+				it != acfg::localdirs.end() ; )
+		{
+			string realSubPath;
+			if(! UrlDecode(tUrl.sPath, realSubPath) || realSubPath.find("..") != stmiss)
+				goto report_invpath; // what the...
+			HandleLocalDownload(sRawUriPath, it->second, realSubPath);
+			return;
+		}
+
+		// entered directory but not defined as local? Then 404 it with hints
 		if(!sPath.empty() && endsWithSzAr(sPath, "/"))
 		{
 			LOG("generic user information page");
@@ -326,19 +558,18 @@ void job::PrepareDownload() {
 			return;
 		}
 
-
 		m_type = rechecks::GetFiletype(sPath);
 
 		if ( m_type == rechecks::FILE_INVALID ) goto report_notallowed;
 		
 		// got something valid, has type now, trace it
-		USRDBG(REPLEVEL, string("Processing new job, ")+m_pReqHead->frontLine);
+		USRDBG(REPLEVEL, "Processing new job, "<<m_pReqHead->frontLine);
 
 		// resolve to an internal location
 		psVname = acfg::GetRepNameAndPathResidual(tUrl, sPathResdiual);
 		
 		if(psVname)
-			m_sFileLoc=*psVname+sPathSep+sPathResdiual;
+			m_sFileLoc=*psVname+SZPATHSEP+sPathResdiual;
 		else
 			m_sFileLoc=tUrl.sHost+tUrl.sPath;
 		
@@ -439,34 +670,34 @@ void job::PrepareDownload() {
 	return;
     
 report_overload:
-    m_sErrorMsgBuffer="503 Server overload, try later"; 
+	SetErrorResponse("503 Server overload, try later");
     return ;
 
 report_notallowed:
-    m_sErrorMsgBuffer="403 Forbidden file type or location";
-    USRDBG(5, m_sErrorMsgBuffer+": "+sRawUriPath);
+	SetErrorResponse("403 Forbidden file type or location");
+    USRDBG(5, sRawUriPath + " -- ACCESS FORBIDDEN");
     return ;
 
 report_offlineconf:
-	m_sErrorMsgBuffer="503 Unable to download in offline mode";
+	SetErrorResponse("503 Unable to download in offline mode");
 	return;
 
 report_invpath:
-    m_sErrorMsgBuffer="403 Invalid path specification"; 
+	SetErrorResponse("403 Invalid path specification");
     return ;
 
 report_invport:
-    m_sErrorMsgBuffer="403 Invalid or prohibited port"; 
+	SetErrorResponse("403 Invalid or prohibited port");
     return ;
 
 report_doubleproxy:
-    m_sErrorMsgBuffer="403 URL seems to be made for proxy but contains apt-cacher-ng port. "
-    		"Inconsistent apt configuration?";
+	SetErrorResponse("403 URL seems to be made for proxy but contains apt-cacher-ng port. "
+    		"Inconsistent apt configuration?");
     return ;
 
 }
 
-#define THROW_ERROR(x) { m_sErrorMsgBuffer=x; goto THROWN_ERROR; }
+#define THROW_ERROR(x) { if(m_nAllDataCount) return R_DISCON; SetErrorResponse(x); return R_AGAIN; }
 job::eJobResult job::SendData(int confd, int nAllJobCount)
 {
 	LOGSTART("job::SendData");
@@ -479,15 +710,12 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 	
 	off_t nGooddataSize(0);
 	FiStatus fistate(FIST_ERROR);
+	header respHead; // template of the response header, e.g. as found in cache
 
-	if (confd<0|| m_state==STATE_FAILMODE)
+	if (confd<0)
 		return R_DISCON; // shouldn't be here
 
-	if (!m_sErrorMsgBuffer.empty()) 
-	{  // should just report it
-		m_state=STATE_FAILMODE;
-	}
-	else if (m_pItem)
+	if (m_pItem)
 	{
 		lockguard g(*m_pItem);
 		
@@ -498,9 +726,18 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 			dbgline;
 			if (fistate>=FIST_ERROR)
 			{
-				const string & sErLine = m_pItem->GetHeaderUnlocked()->frontLine;
-				m_sErrorMsgBuffer = (sErLine.length()<10) ? "500 Unknown error" : sErLine.substr(9);
-				return R_AGAIN; // to send the error response
+				const header &h = m_pItem->GetHeaderUnlocked();
+				g.unLock(); // item lock must be released in order to replace it!
+				if(m_nAllDataCount)
+					return R_DISCON;
+				if(h.h[header::XORIG])
+					m_sOrigUrl=h.h[header::XORIG];
+				// must be something like "HTTP/1.1 403 Forbidden"
+				if(h.frontLine.length()>9 && h.getStatus()!=200)
+					SetErrorResponse(h.frontLine.c_str()+9);
+				else // good ungood? confused somewhere?!
+					SetErrorResponse("500 Unknown error");
+				return R_AGAIN;
 			}
 			
 			dbgline;
@@ -513,9 +750,10 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 			dbgline;
 		}
 		
-		// left loop with usefull state of fileitem, get its header once
-		if(m_RespHead.type == header::INVALID)
-			m_RespHead = *(m_pItem->GetHeaderUnlocked());
+		respHead = m_pItem->GetHeaderUnlocked();
+
+		if(respHead.h[header::XORIG])
+			m_sOrigUrl=respHead.h[header::XORIG];
 
 	}
 	else if(m_state != STATE_SEND_BUFFER)
@@ -524,23 +762,26 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 		return R_DISCON;
 	}
 
+	USRDBG(10, "set/restoring state, fileitem " << size_t(m_pItem.get())
+			<< " with fistate " << fistate << " and procstate " << m_state );
+
 	while (true) // left by returning
 	{
 		MYTRY // for bad_alloc in members
 		{
 			switch(m_state)
 			{
-				case(STATE_FRESH):
+				case(STATE_SEND_MAIN_HEAD):
 				{
 					ldbg("STATE_FRESH");
-					const char *szTempErr(0);
-					eJobResult tempRes = BuildAndEnqueHeader(szTempErr, fistate, nGooddataSize);
-					if(tempRes!=R_UNDEFINED)
-						return tempRes;
-					if(szTempErr)
-						THROW_ERROR(szTempErr);
-					USRDBG(REPLEVEL, "Prepared response header for user: \n" + m_sPrependBuf);
-					continue;
+					if(fistate<FIST_DLGOTHEAD) // be sure about that
+						return R_AGAIN;
+					m_state=STATE_SEND_BUFFER;
+					m_backstate=STATE_HEADER_SENT; // could be changed while creating header
+					const char *szErr = BuildAndEnqueHeader(fistate, nGooddataSize, respHead);
+					if(szErr) THROW_ERROR(szErr);
+					USRDBG(REPLEVEL, "Prepared response header for user: \n" << m_sendbuf );
+					return R_AGAIN;
 				}
 				case(STATE_HEADER_SENT):
 				{
@@ -552,9 +793,8 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 						return R_AGAIN;
 					}
 
-						m_filefd=m_pItem->GetFileFd();
-						if(m_filefd<0)
-							THROW_ERROR("503 IO error");
+					m_filefd=m_pItem->GetFileFd();
+					if(!IsValidFD(m_filefd)) THROW_ERROR("503 IO error");
 
 					m_state=m_bChunkMode ? STATE_SEND_CHUNK_HEADER : STATE_SEND_PLAIN_DATA;
 					ldbg("next state will be: " << m_state);
@@ -594,22 +834,13 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 				}
 				case(STATE_SEND_CHUNK_HEADER):
 				{
-
 					m_nChunkRemainingBytes=nGooddataSize-m_nSendPos;
-
 					ldbg("STATE_SEND_CHUNK_HEADER for " << m_nChunkRemainingBytes);
-					// if not on EOF then the chunk must have remaining size (otherwise the state would have been changed)
-					//ASSERT( 0==(s&FIST_EOF) || 0==m_nChunkRemainingBytes);
-
-					char buf[23];
-					snprintf(buf, 23, "%x\r\n", m_nChunkRemainingBytes);
-					m_sPrependBuf=buf;
-
-					if(m_nChunkRemainingBytes==0)
-						m_sPrependBuf+="\r\n";
+					m_sendbuf << tSS::hex << m_nChunkRemainingBytes << tSS::dec
+							<< (m_nChunkRemainingBytes ? "\r\n" : "\r\n\r\n");
 
 					m_state=STATE_SEND_BUFFER;
-					m_backstate=STATE_SEND_CHUNK_DATA;
+					m_backstate=m_nChunkRemainingBytes ? STATE_SEND_CHUNK_DATA : STATE_FINISHJOB;
 					continue;
 				}
 				case(STATE_SEND_CHUNK_DATA):
@@ -617,7 +848,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 					ldbg("STATE_SEND_CHUNK_DATA");
 
 					if(m_nChunkRemainingBytes==0)
-						GOTOENDE; // yeah...
+						GOTOENDE; // done
 					int n = m_pItem->SendData(confd, m_filefd, m_nSendPos, m_nChunkRemainingBytes);
 					if(n<0)
 						THROW_ERROR("400 Client error");
@@ -625,7 +856,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 					m_nAllDataCount+=n;
 					if(m_nChunkRemainingBytes<=0)
 					{ // append final newline
-						m_sPrependBuf="\r\n";
+						m_sendbuf<<"\r\n";
 						m_state=STATE_SEND_BUFFER;
 						m_backstate=STATE_SEND_CHUNK_HEADER;
 						continue;
@@ -634,30 +865,23 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 				}
 				case(STATE_SEND_BUFFER):
 				{
-          // Sends data in the local buffer, used for page or chunk headers
-          // sending, also error header sending. -> DELICATE STATE, should not
-          // be interrupted by exception or throw to another flow uncontrolled.
+					/* Sends data from the local buffer, used for page or chunk headers.
+					* -> DELICATE STATE, should not be interrupted by exception or throw
+					* to another state path uncontrolled. */
 					MYTRY
 					{
-						ldbg("prebuf sende: "<< m_sPrependBuf);
-						int r=send(confd, m_sPrependBuf.data(), m_sPrependBuf.length(), 
-								(m_backstate == STATE_ERRORCONT || m_backstate == STATE_TODISCON) ? 0 : MSG_MORE);
+						ldbg("prebuf sending: "<< m_sendbuf.c_str());
+						int r=send(confd, m_sendbuf.rptr(), m_sendbuf.size(),
+								m_backstate == STATE_TODISCON ? 0 : MSG_MORE);
 						if (r<0)
 						{
 							if (errno==EAGAIN || errno==EINTR || errno == ENOBUFS)
-							{
 								return R_AGAIN;
-							}
 							return R_DISCON;
 						}
 						m_nAllDataCount+=r;
-						m_sPrependBuf.erase(0, r);
-						
-						USRDBG(REPLEVEL, "Sent " << r 
-								<< " bytes of header data, remaining contents\n" 
-								<< m_sPrependBuf);
-						
-						if(m_sPrependBuf.empty())
+						m_sendbuf.drop(r);
+						if(m_sendbuf.empty())
 						{
 							USRDBG(REPLEVEL, "Returning to last state, " << m_backstate);
 							m_state=m_backstate;
@@ -671,9 +895,6 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 					return R_AGAIN;
 				}
 				
-				case (STATE_FAILMODE):
-					goto THROWN_ERROR;
-				
 				case(STATE_ALLDONE):
 					LOG("State: STATE_ALLDONE?");
 				case (STATE_ERRORCONT):
@@ -681,17 +902,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 				case(STATE_FINISHJOB):
 					LOG("or STATE_FINISHJOB");
 					{
-						bool bClose=true;
-						if(m_pReqHead)
-						{
-							bool bhttp11 = m_pReqHead->frontLine.find("HTTP/1.1") != stmiss;
-							if(m_pReqHead->h[header::CONNECTION])
-								bClose = 0==strcasecmp(m_pReqHead->h[header::CONNECTION], "close");
-							else
-								bClose = !bhttp11;
-						}
-
-						if(bClose)
+						if(m_bClientWants2Close)
 							return R_DISCON;
 						LOG("Reporting job done")
 						return R_DONE;
@@ -701,132 +912,95 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 				default:
 					return R_DISCON;
 			}
-			
-			continue;
-			
-			
-	THROWN_ERROR:
-			ldbg("Processing error: " << m_sErrorMsgBuffer);
-			if(0==m_nAllDataCount) // don't send errors when data header is underway
-			{
-				m_sPrependBuf=string("HTTP/1.1 ")+m_sErrorMsgBuffer+"\r\n";
-				
-				// only respond if the client has set it explicitely
-				if(m_pReqHead->h[header::CONNECTION])
-				{
-					// whatever the client wants
-					m_sPrependBuf+="Connection: ";
-					m_sPrependBuf+=m_pReqHead->h[header::CONNECTION];
-					m_sPrependBuf+="\r\n";
-				}
-
-				m_sPrependBuf+=header::GetInfoHeaders();
-						
-				if(m_RespHead.h[header::XORIG])
-				{
-					m_sPrependBuf+="X-Original-Source: ";
-					m_sPrependBuf+=m_RespHead.h[header::XORIG];
-					m_sPrependBuf+="\r\n";
-				}
-				else if(!m_sFileLoc.empty())
-				{
-					m_sPrependBuf+="X-Original-Source: ";
-					m_sPrependBuf+=m_sFileLoc+"\r\n";
-				}
-				
-				m_sPrependBuf+="\r\n";
-				m_backstate=STATE_ERRORCONT;
-				m_sErrorMsgBuffer.clear();
-				USRDBG(REPLEVEL, "Response error or message, will continue (for " << m_sFileLoc<<")");
-				//USRDBG(REPLEVEL, h.as_string(true));
-				m_state=STATE_SEND_BUFFER;
-
-				continue;
-			}
-			ldbg("Headers already sent -> forcing abort")
-			return R_DISCON;
 		}
 		MYCATCH(bad_alloc) {
 			// TODO: report memory failure?
 			return R_DISCON;
 		}
-		ASSERT(!"UNREACHED");
+		//ASSERT(!"UNREACHED");
 	}
 }
 
-#define REPORT_ERR(x) { szErrRet=x; return R_UNDEFINED; }
-inline job::eJobResult job::BuildAndEnqueHeader(const char * &szErrRet, const FiStatus &fistate, const off_t &nGooddataSize)
+
+inline const char * job::BuildAndEnqueHeader(const FiStatus &fistate,
+		const off_t &nGooddataSize, header& respHead)
 {
-	if(fistate<FIST_DLGOTHEAD) // be sure about that
-		return R_AGAIN;
+	LOGSTART("job::BuildAndEnqueHeader");
 
-	m_state=STATE_SEND_BUFFER;
-	m_backstate=STATE_HEADER_SENT; // might change without body
+	if(respHead.type != header::ANSWER)
+		return "500 Rotten Data";
 
-	if(m_RespHead.type != header::ANSWER)
-		REPORT_ERR("500 Rotten Data");
+#if 0 // defined DEBUG
+	respHead.h[header::CONTENT_LENGTH]=0;
+	respHead.set(header::TRANSFER_ENCODING, "chunked");
+#endif
 
-	//h.del("Content-Length"); // TESTCASE: Force chunked mode
-
-	m_sPrependBuf=m_RespHead.frontLine+"\r\n";
-
-	bool bSendBody(true);
-	// no data for error heads
-	if(m_RespHead.getStatus() != 200)
-		bSendBody=false;
-
-	// make sure there is no junk inside
-	// if missing completely, chunked mode is to be used below
-	if(m_RespHead.h[header::CONTENT_LENGTH])
+	// make sure that header has consistent state and there is data to send which is expected by the client
+	bool bHasSendableData = false;
+	int httpstatus = respHead.getStatus();
+	LOG("State: " << httpstatus);
+	if (304 != httpstatus) // never for not-unmodified
 	{
-		if(0==atoofft(m_RespHead.h[header::CONTENT_LENGTH]))
-			bSendBody=false;
-
-		/*
-		if(!bSendBody && ! m_pReqHead->type==header::HEAD)
-			m_RespHead.del(header::CONTENT_LENGTH);
-			*/
+		bool bChunked = respHead.h[header::TRANSFER_ENCODING] && 0 == strncasecmp( respHead.h[header::TRANSFER_ENCODING], "chunk", 5);
+		if (httpstatus == 200)
+		{
+			if (bChunked)
+			{
+				// chunked data, usually ok...
+				if (respHead.h[header::CONTENT_LENGTH])
+				{
+					aclog::err("internal error: chunked transfer encoding and content length value are both set");
+					respHead.del(header::CONTENT_LENGTH);
+				}
+				bHasSendableData = true;
+			}
+			else
+				bHasSendableData	= respHead.h[header::CONTENT_LENGTH];
+			LOG("has contlen? " << bool(respHead.h[header::CONTENT_LENGTH]));
+		}
+		else if (httpstatus >= 300)
+			bHasSendableData = (respHead.h[header::CONTENT_LENGTH] || bChunked);
 	}
 
-	if( ! bSendBody)
-	{
-		m_backstate=STATE_ALLDONE;
-	}
-	else
-	{
+	tSS &sb=m_sendbuf;
+	sb.clear();
+	sb << respHead.frontLine <<"\r\n";
 
-		if( ! m_RespHead.h[header::CONTENT_LENGTH])
+	bool bGotLen(false);
+
+	if(bHasSendableData)
+	{
+		LOG("has sendable content");
+		if( ! respHead.h[header::CONTENT_LENGTH])
 		{
 			// unknown length but must have data, will have to improvise: prepare chunked transfer
-			if(m_bIsHttp11 != 0)
-			REPORT_ERR("505 HTTP version not supported for this file"); // you don't accept this, go away
+			if ( ! m_bIsHttp11 ) // you cannot process this? go away
+				return "505 HTTP version not supported for this file";
 			m_bChunkMode=true;
-			m_RespHead.set(header::TRANSFER_ENCODING, "chunked");
+			sb<<"Transfer-Encoding: chunked\r\n";
 		}
-		else
+		else if(200==httpstatus) // good data response with known length, can try some optimizations
 		{
-			// OK, has content length. Can do optimizations?
-			// Handle If-Modified-Since and Range headers
+			LOG("has known content length, optimizing response...");
 
+			// Handle If-Modified-Since and Range headers;
 			// we deal with them equally but need to know which to use
 			const char *pIfmo = m_pReqHead->h[header::RANGE] ?
 					m_pReqHead->h[header::IFRANGE] : m_pReqHead->h[header::IF_MODIFIED_SINCE];
 
-			bool bGotLen(false);
-
-			// consider contents "fresh" for non-volatile data in "special cases"
-			bool bFreshnessForced=(m_type != rechecks::FILE_INDEX
-					|| m_pReqHead->h[header::ACNGFSMARK]);
+			// consider contents "fresh" for non-volatile data, or when "our" special client is there, or the client simply doesn't care
+			bool bFreshnessForced = (m_type != rechecks::FILE_INDEX
+				|| m_pReqHead->h[header::ACNGFSMARK] || !pIfmo);
 
 			// this plays well with lighttpd <-> Apt, but may not work with the alternative time format
 			// TODO: add format tolerant time comparer
-			bool bIfModSeenAndChecked=(pIfmo && m_RespHead.h[header::LAST_MODIFIED] &&
-					0==strcmp(pIfmo, m_RespHead.h[header::LAST_MODIFIED]));
+			bool bIfModSeenAndChecked=(pIfmo && respHead.h[header::LAST_MODIFIED] &&
+					0==strcmp(pIfmo, respHead.h[header::LAST_MODIFIED]));
 
 			// is it fresh? or is this relevant? or is range mode forced?
 			if(  bFreshnessForced || bIfModSeenAndChecked)
 			{
-				off_t nContLen=atoofft(m_RespHead.h[header::CONTENT_LENGTH]);
+				off_t nContLen=atoofft(respHead.h[header::CONTENT_LENGTH]);
 
 				/*
 				 * Range: bytes=453291-
@@ -857,12 +1031,10 @@ inline job::eJobResult job::BuildAndEnqueHeader(const char * &szErrRet, const Fi
 						nRangeTo=nContLen-1;
 
 					/*
-					 * make sure that our client doesn't just
-					 * hang here while the download thread is
-					 * fetching from 0 to start position for
-					 * many minutes
+					 * make sure that our client doesn't just hang here while the download thread is
+					 * fetching from 0 to start position for many minutes. If the resumed position
+					 * is beyond of what we already have, fall back to 200 (complete download).
 					 */
-
 					if(nRangeItems>0
 						&&
 						(  fistate==FIST_COMPLETE
@@ -877,84 +1049,108 @@ inline job::eJobResult job::BuildAndEnqueHeader(const char * &szErrRet, const Fi
 							)
 						)
 					{
+						USRDBG(20, "request range problem: ")
 						// detect errors, out-of-range case
 						if(nRangeTo>=nContLen || nRangeFrom>=nContLen || nRangeTo<nRangeFrom)
-							REPORT_ERR("416 Requested Range Not Satisfiable")
+							return "416 Requested Range Not Satisfiable";
 
 						m_nSendPos = nRangeFrom;
 						m_nRangeLast = nRangeTo;
-						m_sPrependBuf="HTTP/1.1 206 Partial Response\r\nContent-Length: ";
-						char buf[200];
-						sprintf(buf,
-                                OFF_T_FMT "\r\n"
-                                "Content-Range: bytes " OFF_T_FMT "-" OFF_T_FMT "/" OFF_T_FMT "\r\n",
-								m_nRangeLast-m_nSendPos+1,
-								m_nSendPos,
-								m_nRangeLast,
-								nContLen
-								);
-						m_sPrependBuf+=buf;
+						// replace with partial-response header
+						sb.clear();
+						sb << "HTTP/1.1 206 Partial Response\r\nContent-Length: "
+						 << (m_nRangeLast-m_nSendPos+1) <<
+								"\r\nContent-Range: bytes "<< m_nSendPos
+								<< "-" << m_nRangeLast << "/" << nContLen << "\r\n";
 						bGotLen=true;
 					}
 				}
 				else if(bIfModSeenAndChecked)
 				{
 					// file is fresh, and user sent if-mod-since -> fine
-					REPORT_ERR("304 Not Modified");
+					return "304 Not Modified";
 				}
 			}
-			// has cont-len available but this header was not set yet in the code above
-			if( !bGotLen)
-			{
-				m_sPrependBuf+=string("Content-Length: ")
-				+m_RespHead.h[header::CONTENT_LENGTH]
-				+"\r\n";
-			}
 		}
-		if(m_RespHead.h[header::LAST_MODIFIED])
-			m_sPrependBuf+=string("Last-Modified: ")
-			+m_RespHead.h[header::LAST_MODIFIED]+"\r\n";
+		// has cont-len available but this header was not set yet in the code above
+		if( !bGotLen && !m_bChunkMode)
+			sb<<"Content-Length: "<<respHead.h[header::CONTENT_LENGTH]<<"\r\n";
+
+		// OK, has data for user and has set content-length and/or range or chunked transfer mode, now add various meta headers...
+
+		if(respHead.h[header::LAST_MODIFIED])
+			sb<<"Last-Modified: "<<respHead.h[header::LAST_MODIFIED]<<"\r\n";
+
+		sb<<"Content-Type: ";
+		if(respHead.h[header::CONTENT_TYPE])
+			sb<<respHead.h[header::CONTENT_TYPE]<<"\r\n";
+		else
+			sb<<"application/octet-stream\r\n";
 	}
-	m_sPrependBuf+=header::GetInfoHeaders();
+	else
+		m_backstate=STATE_ALLDONE;
 
-	if(bSendBody)
-		m_sPrependBuf+="Content-Type: application/octet-stream\r\n";
-	else if(m_RespHead.h[header::LOCATION])
-	{
-			m_sPrependBuf+="Location: ";
-			m_sPrependBuf+=m_RespHead.h[header::LOCATION];
-			m_sPrependBuf+="\r\n";
-	}
+	sb << header::GenInfoHeaders();
 
-	if(m_RespHead.h[header::XORIG])
-		m_sPrependBuf+=string("X-Original-Source: ")+m_RespHead.h[header::XORIG]+"\r\n";
+	if(respHead.h[header::LOCATION])
+		sb<<"Location: "<<respHead.h[header::LOCATION]<<"\r\n";
 
-	// only respond if the client has set it explicitely
-	if(m_pReqHead->h[header::CONNECTION])
-	{
-		// whatever the client wants
-		m_sPrependBuf+="Connection: ";
-		m_sPrependBuf+=m_pReqHead->h[header::CONNECTION];
-		m_sPrependBuf+="\r\n";
-	}
+	if(!m_sOrigUrl.empty())
+		sb<<"X-Original-Source: "<< m_sOrigUrl <<"\r\n";
 
-	m_sPrependBuf+="\r\n";
+	// whatever the client wants
+	sb<<"Connection: "<<(m_bClientWants2Close?"close":"Keep-Alive")<<"\r\n";
+
+	sb<<"\r\n";
+	LOG("response prepared:" << sb);
 
 	if(m_pReqHead->type==header::HEAD)
 		m_backstate=STATE_ALLDONE; // simulated head is prepared but don't send stuff
 
-
-	if(m_nSendPos>0) // maybe needs to wait in the prerequisites check in the beginning
-		return R_AGAIN;
-
-	return R_UNDEFINED;
+	return 0;
 }
 
-inline FiStatus job::_SwitchToPtItem(const MYSTD::string &fileLoc)
+FiStatus job::_SwitchToPtItem(const MYSTD::string &fileLoc)
 {
+	// Changing to local pass-through file item
 	LOGSTART("job::_SwitchToPtItem");
-	LOG("Changing to local pass-through file item");
-	m_pItem->Unreg();
+	// exception-safe sequence
+	tFileItemPtr tmp=m_pItem;
+	m_pItem.reset();
+	tmp->Unreg();
 	m_pItem.reset(new tPassThroughFitem(m_sFileLoc));
 	return m_pItem->Setup(true);
 }
+
+void job::SetErrorResponse(const char * errorLine, int nMinUserLogLevel)
+{
+	LOGSTART2("job::SetErrorResponse", errorLine);
+
+
+	class erroritem: public tGeneratedFitemBase
+	{
+	public:
+		erroritem(const string &sId, const char *szError) : tGeneratedFitemBase(sId, szError)
+		{
+			// not-modified -> no response
+			if(304 == m_head.getStatus())
+				return;
+			// otherwise do something meaningful
+			m_data << "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\n"
+				"<html><head><title>" << szError << "</title>\n</head>\n<body><h1>"
+				<< szError << "</h1></body></html>";
+			m_head.set(header::CONTENT_TYPE, "text/html");
+			seal();
+		}
+	};
+
+	tFileItemPtr pNew(static_cast<fileitem*>(new erroritem("noid", errorLine)));
+
+	if(m_pItem)
+		m_pItem->Unreg();
+
+	m_pItem=pNew;
+
+	//aclog::err(tSS() << "fileitem is now " << uintptr_t(m_pItem.get()));
+	m_state=STATE_SEND_MAIN_HEAD;
+}
