@@ -7,7 +7,6 @@
 #include "expiration.h"
 #include "pkgimport.h"
 #include "showinfo.h"
-#include "reportgen.h"
 #include "aclogger.h"
 #include "filereader.h"
 #include "acfg.h"
@@ -36,14 +35,6 @@ maintenance::~maintenance()
 {
 }
 
-/// @brief Sends a HTTP chunk with string contents
-/// To terminate the transfer, add an empty string with force flag
-
-void maintenance::SendChunk(const string & x) 
-{
-    SendChunk(x.data(), x.length());
-}
-
 bool maintenance::SendRawData(const char *data, size_t len, int flags)
 {
 	while(len>0)
@@ -67,7 +58,7 @@ void maintenance::SendChunk(const char *data, size_t len)
 {
 	if(!data || !len || m_reportFD<0)
 		return;
-	
+
 	char buf[23];
 	int l=sprintf(buf, "%x\r\n", (UINT) len);
 	SendRawData(buf, l, MSG_MORE|MSG_NOSIGNAL);
@@ -77,19 +68,17 @@ void maintenance::SendChunk(const char *data, size_t len)
 
 void maintenance::EndTransfer() 
 {
-	const char msg[] = "0\r\n\r\n"; 
-	SendRawData(msg, sizeof(msg)-1, MSG_NOSIGNAL);
+	SendRawData(_SZ2PS("0\r\n\r\n"), MSG_NOSIGNAL);
 }
 
 void maintenance::SendChunkedPageHeader(const char *httpcode, const char *mimetype)
 {
-	MYSTD::string buf("HTTP/1.1 ");
-	buf+=(httpcode ? httpcode : "200");
-	buf+=" OK\r\nConnection: close\r\n"
-		"Transfer-Encoding: chunked\r\nContent-Type: ";
-	buf+=(mimetype?mimetype:"text/html");
-	buf+="\r\n\r\n";
-	SendRawData(buf.data(), buf.length(), MSG_MORE);
+	tSS s(100);
+	s <<  "HTTP/1.1 " << (httpcode ? httpcode : "200") << " OK\r\n"
+			"Connection: close\r\n"
+			"Transfer-Encoding: chunked\r\n"
+			"Content-Type: " << (mimetype?mimetype:"text/html") << "\r\n\r\n";
+	SendRawData(s.data(), s.length(), MSG_MORE);
 }
 /*
 void maintenance::SendDecoration(bool bBegin, const char *szDecoFile)
@@ -191,7 +180,7 @@ void maintenance::SetStyle(string &s)
 	tStrPos pos=0;
 
 	acbuf buf;
-	if( ! buf.initFromFile( (acfg::confdir+sPathSep+"style.css").c_str()) )
+	if( ! buf.initFromFile( (acfg::confdir+SZPATHSEP "style.css").c_str()) )
 		return;
 	
 	while(stmiss != (pos = s.find("$STYLE", pos)))
