@@ -16,14 +16,10 @@
 #include <glob.h>
 #endif
 
-
 using namespace MYSTD;
 
 string sPathSep(SZPATHSEP);
-string sPathSepUnix(SZPATHSEP);
-const char *szPathSep=SZPATHSEP;
-string sCRLF("\r\n");
-string sLF("\n");
+string sPathSepUnix(SZPATHSEPUNIX);
 
 /*
 int getUUID() {
@@ -284,13 +280,6 @@ string tHttpUrl::ToURI() const
 }
 //tHttpUrl::tHttpUrl() : pMisc(NULL) {};
 
-string ltos(long n)
-{
-	char buf[21];
-	snprintf(buf, 21, "%ld", n);
-	return buf;
-}
-
 /*
 const MYSTD::string GetDateString() {
     char buf[26];
@@ -307,9 +296,9 @@ const MYSTD::string GetDateString() {
 
 #if defined(HAVE_WORDEXP) || defined(HAVE_GLOB)
 
-tStrVec ExpandFilePattern(const string &pattern, bool bSorted)
+tStrDeq ExpandFilePattern(const string &pattern, bool bSorted)
 {
-	tStrVec srcs;
+	tStrDeq srcs;
 #ifdef HAVE_WORDEXP
 	wordexp_t p;
 	memset(&p, 0, sizeof(p));
@@ -337,11 +326,56 @@ tStrVec ExpandFilePattern(const string &pattern, bool bSorted)
 
 	return srcs;
 }
-#else // -> no wordexp, no glob
-tStrVec GetStringAsVector(const string &s)
+
+
+void MakeAbsolutePath(MYSTD::string &dirToFix, const MYSTD::string &reldir)
 {
-	tStrVec ret;
-	ret.push_back(s);
-	return s;
+	bool bAbs=false;
+#ifdef WIN32
+	bAbs=dirToFix.length()> 2 && CPATHSEPWIN==dirToFix[2] && ':'==dirToFix[1] && isalpha(dirToFix[0];
+	if(!bAbs) // maybe unc path?
+		bAbs=(dirToFix[0] == CPATHSEPWIN &&dirToFix[1] == CPATHSEPWIN);
+#else
+	bAbs = (!dirToFix.empty() && CPATHSEPUNX==dirToFix[0]);
+#endif
+	if(!bAbs)
+		dirToFix=reldir+CPATHSEP+dirToFix;
 }
+
+
+void HttpEscape(string &s)
+{
+	tStrPos p;
+	while(stmiss!=(p=s.find_first_of("~ ")))
+	{
+		switch(s[p])
+		{
+		case('~'): s.replace(p, 1, "%7e"); break;
+		case(' '): s.replace(p, 1, "&nbsp"); break;
+		}
+	}
+}
+
+extern uint_fast16_t hexmap[];
+
+bool UrlDecode(const MYSTD::string &from, MYSTD::string & to)
+{
+	bool ret=true;
+	for(string::size_type i=0; i<from.length(); i++)
+	{
+		if(from[i] != '%')
+			to+=from[i];
+		else if(i>from.length()-3) // cannot start another sequence here (too short), keep as-is
+			ret=false;
+		else
+		{
+			to+=char(16*hexmap[(unsigned char) from[i+1]] + hexmap[(unsigned char) from[i+2]]);
+			i+=2;
+		}
+	}
+	return ret;
+}
+
+
+
 #endif
