@@ -18,7 +18,7 @@
 
 using namespace MYSTD;
 
-#define SCACHEFILE (acfg::cachedir+sPathSep+"_impkeycache")
+#define SCACHEFILE (acfg::cachedir+SZPATHSEP"_impkeycache")
 extern bool bSigTaskAbort;
 extern pthread_mutex_t abortMx;
 
@@ -33,7 +33,7 @@ extern pthread_mutex_t abortMx;
  *
 */
 
-pkgimport::pkgimport(int fd) : expiration(fd), m_bPickIfiles(false)
+pkgimport::pkgimport(int fd) : expiration(fd), m_bLookForIFiles(false)
 {
 }
 
@@ -77,20 +77,14 @@ bool pkgimport::ProcessRegular(const string &sPath, const struct stat &stinfo)
 	if(endsWithSzAr(sPath, ".head"))
 		return true;
 	
-	//string sRelName=fileNode->sPath.substr(acfg::cachedir.length()+1);
-	rechecks::eFileKind kind = rechecks::GetFiletype(sPath);
-	if(m_bPickIfiles)
+	if(m_bLookForIFiles)
 	{
 		if(0==sPath.compare(m_nDropPrefLen, 1, "_"))
 			return true; // not for us, also includes _import
 		
-		if( rechecks::FILE_INDEX == kind )
-		{
-			//cout << "Is ifile\n";
-			m_volatileFiles.insert(sPath);
-		}
+		AddIFileCandidate(sPath);
 	}
-	else /* also try to import ifiles when matched, why not?! if (rechecks::FILE_PKG == kind) */
+	else
 	{
 		// get a fingerprint by checksumming if not already there from the fpr cache
 
@@ -232,17 +226,17 @@ bool pkgimport::LinkOrCopy(const MYSTD::string &from, const MYSTD::string &to)
 void pkgimport::Action(const string &cmd) 
 {
 // TODO: import path from cmd?
-	m_sSrcPath=acfg::cachedir+sPathSep+"_import";
+	m_sSrcPath=acfg::cachedir+SZPATHSEP"_import";
 	
 	SendChunk(string("Importing from ")+m_sSrcPath+" directory, scanning...<br>\n");
 	
 	_LoadKeyCache(SCACHEFILE);
 	if(!m_precachedList.empty())
-		SendChunk(string("Loaded ")+ltos(m_importMap.size())
-				+(m_precachedList.size()==1 ? " entry" : " entries")
-				+" from the fingerprint cache<br>\n");
+		SendChunk( tSS(100)<<"Loaded "<<m_importMap.size()<<
+				(m_precachedList.size()==1 ? " entry" : " entries")
+				<<" from the fingerprint cache<br>\n");
 	
-	m_bPickIfiles=true;
+	m_bLookForIFiles=true;
 	DirectoryWalk(acfg::cachedir, this);
 
 	{
@@ -251,13 +245,13 @@ void pkgimport::Action(const string &cmd)
 			return;
 	}
 	
-	if(m_volatileFiles.empty())
+	if(m_indexFiles.empty())
 	{
 		SendChunk("<font size=0 color=red>No index files detected. Unable to continue, cannot map files to internal locations.</font>");
 		return;
 	}
 
-	_UpdateVolatileFiles();
+	UpdateIndexFiles();
 
 	{
 		lockguard g(&abortMx);
@@ -271,7 +265,7 @@ void pkgimport::Action(const string &cmd)
 		return;
 	}
 	
-	m_bPickIfiles=false;
+	m_bLookForIFiles=false;
 	DirectoryWalk(m_sSrcPath, this);
 
 	{
@@ -286,7 +280,7 @@ void pkgimport::Action(const string &cmd)
 		return;
 	}
 	
-	_ParseVolatileFilesAndHandleEntries();
+	_ProcessSeenIndexFiles();
 
 	{
 		lockguard g(&abortMx);
@@ -328,37 +322,6 @@ void pkgimport::Action(const string &cmd)
 	checkForceFclose(cfh);
 }
 
-#if 0
-void HttpEscape(string &s)
-{
-	tStrPos p;
-	while(stmiss!=(p=s.find_first_of("~ ")))
-	{
-		switch(s[p])
-		{
-		case('~'): s.replace(p, 1, "%7e"); break;
-		case(' '): s.replace(p, 1, "&nbsp"); break;
-		}
-	}
-}
-
-extern uint_fast16_t hexmap[];
-
-inline void UrlDecode(string &s)
-{
-	for(string::size_type i=0; i<s.length(); i++)
-	{
-		if(s[i]!='%')
-			continue;
-		if(i>s.length()-3) // cannot start another sequence here (too short), keep as-is
-			continue;
-		char decoded=16*hexmap[(unsigned char) s[i+1]] + hexmap[(unsigned char) s[i+2]];
-		s.replace(i, 3, 1, decoded);
-	}
-}
-
-#endif
-
 void pkgimport::_HandlePkgEntry(const tRemoteFileInfo &entry)
 {
 	
