@@ -139,7 +139,7 @@ bool tcpconnect::_Connect(const string & sHostname, string & sErrorMsg)
 	// format the last available error message for the user
 	aclog::errnoFmter fer;
 	sErrorMsg="500 Connection failure: ";
-	sErrorMsg+=fer.msg;
+	sErrorMsg+=fer;
 	ldbg("Force reconnect, con. failure");
 	_Disconnect();
 	return false;
